@@ -12,7 +12,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02111-1301  USA
  */
 #include <config.h>
 
@@ -28,6 +28,7 @@
 #include <malloc.h>
 #include <ctype.h>
 #include "error.h"
+#include "ignore.h"
 #include "strlist.h"
 #include "md5sum.h"
 #include "names.h"
@@ -37,6 +38,7 @@
 #include "signature.h"
 #include "sources.h"
 #include "files.h"
+#include "tracking.h"
 #include "guesscomponent.h"
 #include "override.h"
 #include "checkindsc.h"
@@ -85,6 +87,8 @@ struct fileentry {
 	char *component;
 	/* only set after changes_includefiles */
 	char *filekey;
+	/* set between checkpkg and includepkg */
+	union { struct dscpackage *dsc; struct debpackage *deb;} pkg;
 };
 
 struct changes {
@@ -98,6 +102,8 @@ struct changes {
 	/* Things to be set by changes_fixfields: */
 	/* the component source files are put into */
 	const char *srccomponent;
+	/* != NULL if changesfile was put into pool/ */
+	/*@null@*/ char *changesfilekey;
 	/* the directory where source files are put into */
 	char *srcdirectory;
 	/* (only to warn if multiple are used) */
@@ -107,7 +113,7 @@ struct changes {
 static void freeentries(/*@only@*/struct fileentry *entry) {
 	struct fileentry *h;
 
-	while( entry ) {
+	while( entry != NULL ) {
 		h = entry->next;
 		free(entry->filekey);
 		free(entry->component);
@@ -117,6 +123,10 @@ static void freeentries(/*@only@*/struct fileentry *entry) {
 		free(entry->priority);
 		free(entry->architecture);
 		free(entry->name);
+		if( entry->type == fe_DEB || entry->type == fe_UDEB)
+			deb_free(entry->pkg.deb);
+		 else if( entry->type == fe_DSC )
+			dsc_free(entry->pkg.dsc);
 		free(entry);
 		entry = h;
 	}
@@ -132,6 +142,8 @@ static void changes_free(/*@only@*/struct changes *changes) {
 		strlist_done(&changes->distributions);
 		free(changes->control);
 		free(changes->srcdirectory);
+		free(changes->changesfilekey);
+//		trackedpackage_free(changes->trackedpkg);
 	}
 	free(changes);
 }
@@ -149,37 +161,37 @@ static retvalue newentry(struct fileentry **entry,const char *fileline,const cha
 	filetype type;
 
 	p = fileline;
-	while( *p !='\0' && isspace(*p) )
+	while( *p !='\0' && xisspace(*p) )
 		p++;
 	md5start = p;
-	while( *p !='\0' && !isspace(*p) )
+	while( *p !='\0' && !xisspace(*p) )
 		p++;
 	md5end = p;
-	while( *p !='\0' && isspace(*p) )
+	while( *p !='\0' && xisspace(*p) )
 		p++;
 	sizestart = p;
-	while( *p !='\0' && !isspace(*p) )
+	while( *p !='\0' && !xisspace(*p) )
 		p++;
 	sizeend = p;
-	while( *p !='\0' && isspace(*p) )
+	while( *p !='\0' && xisspace(*p) )
 		p++;
 	sectionstart = p;
-	while( *p !='\0' && !isspace(*p) )
+	while( *p !='\0' && !xisspace(*p) )
 		p++;
 	sectionend = p;
-	while( *p !='\0' && isspace(*p) )
+	while( *p !='\0' && xisspace(*p) )
 		p++;
 	priostart = p;
-	while( *p !='\0' && !isspace(*p) )
+	while( *p !='\0' && !xisspace(*p) )
 		p++;
 	prioend = p;
-	while( *p !='\0' && isspace(*p) )
+	while( *p !='\0' && xisspace(*p) )
 		p++;
 	filestart = p;
-	while( *p !='\0' && !isspace(*p) )
+	while( *p !='\0' && !xisspace(*p) )
 		p++;
 	fileend = p;
-	while( *p !='\0' && isspace(*p) )
+	while( *p !='\0' && xisspace(*p) )
 		p++;
 	if( *p != '\0' ) {
 		fprintf(stderr,"Unexpected sixth argument in '%s'!\n",fileline);
@@ -192,7 +204,7 @@ static retvalue newentry(struct fileentry **entry,const char *fileline,const cha
 	}
 
 	p = filestart;
-	while( *p != '\0' && *p != '_' && !isspace(*p) )
+	while( *p != '\0' && *p != '_' && !xisspace(*p) )
 		p++;
 	if( *p != '_' ) {
 		if( *p == '\0' )
@@ -226,7 +238,7 @@ static retvalue newentry(struct fileentry **entry,const char *fileline,const cha
 		archend = p;
 		p++;
 		typestart = p;
-		while( *p !='\0' && !isspace(*p) )
+		while( *p !='\0' && !xisspace(*p) )
 			p++;
 		if( p-typestart == 3 && strncmp(typestart,"deb",3) == 0 )
 			type = fe_DEB;
@@ -243,7 +255,7 @@ static retvalue newentry(struct fileentry **entry,const char *fileline,const cha
 	} else {
 		/* this looks like some source-package, we will have
 		 * to look for the packagetype ourself... */
-		while( *p !='\0' && !isspace(*p) ) {
+		while( *p !='\0' && !xisspace(*p) ) {
 			p++;
 		}
 		if( p-versionstart > 12 && strncmp(p-12,".orig.tar.gz",12) == 0 )
@@ -283,11 +295,12 @@ static retvalue newentry(struct fileentry **entry,const char *fileline,const cha
 	e->name = strndup(filestart,nameend-filestart);
 	e->type = type;
 
-	if( !e->basename || !e->md5sum || !e->section || !e->priority || !e->architecture || !e->name ) {
+	if( e->basename == NULL || e->md5sum == NULL || e->section == NULL || 
+	    e->priority == NULL || e->architecture == NULL || e->name == NULL ) {
 		freeentries(e);
 		return RET_ERROR_OOM;
 	}
-	if( forcearchitecture ) {
+	if( forcearchitecture != NULL ) {
 		if( strcmp(forcearchitecture,"source") != 0 && 
 				strcmp(e->architecture,"all") == 0 ) {
 			if( verbose > 2 )
@@ -334,26 +347,29 @@ static retvalue changes_parsefilelines(const char *filename,struct changes *chan
 	return result;
 }
 
-static retvalue check(const char *filename,struct changes *changes,const char *field,int force) {
+static retvalue check(const char *filename,struct changes *changes,const char *field) {
 	retvalue r;
 
 	r = chunk_checkfield(changes->control,field);
 	if( r == RET_NOTHING ) {
-		fprintf(stderr,"In '%s': Missing '%s' field!\n",filename,field);
-		if( !force )
+		if( IGNORING("Ignoring","To Ignore",missingfield,
+				"In '%s': Missing '%s' field!\n",filename,field) ) {
+			return RET_OK;
+		} else {
 			return RET_ERROR;
+		} 
 	}
 	return r;
 }
 
-static retvalue changes_read(const char *filename,/*@out@*/struct changes **changes,/*@null@*/const char *packagetypeonly,/*@null@*/const char *forcearchitecture,int force, bool_t onlysigned) {
+static retvalue changes_read(const char *filename,/*@out@*/struct changes **changes,/*@null@*/const char *packagetypeonly,/*@null@*/const char *forcearchitecture, bool_t onlysigned) {
 	retvalue r;
 	struct changes *c;
 	struct strlist filelines;
 
-#define E(err,param...) { \
+#define E(err) { \
 		if( r == RET_NOTHING ) { \
-			fprintf(stderr,"In '%s': " err "\n",filename , ## param ); \
+			fprintf(stderr,"In '%s': " err "\n",filename); \
 			r = RET_ERROR; \
 	  	} \
 		if( RET_WAS_ERROR(r) ) { \
@@ -361,17 +377,6 @@ static retvalue changes_read(const char *filename,/*@out@*/struct changes **chan
 			return r; \
 		} \
 	}
-#define C(err,param...) { \
-		if( RET_WAS_ERROR(r) ) { \
-			if( !force ) { \
-				fprintf(stderr,"In '%s': " err "\n",filename , ## param ); \
-				changes_free(c); \
-				return r; \
-			} else { \
-				fprintf(stderr,"Ignoring " err " in '%s' due to --force:\n " err "\n" , ## param , filename); \
-			} \
-		} \
-	}
 #define R { \
 		if( RET_WAS_ERROR(r) ) { \
 			changes_free(c); \
@@ -385,9 +390,9 @@ static retvalue changes_read(const char *filename,/*@out@*/struct changes **chan
 		return RET_ERROR_OOM;
 	r = signature_readsignedchunk(filename,&c->control,onlysigned);
 	R;
-	r = check(filename,c,"Format",force);
+	r = check(filename,c,"Format");
 	R;
-	r = check(filename,c,"Date",force);
+	r = check(filename,c,"Date");
 	R;
 	r = chunk_getname(c->control,"Source",&c->source,FALSE);
 	E("Missing 'Source' field");
@@ -403,13 +408,13 @@ static retvalue changes_read(const char *filename,/*@out@*/struct changes **chan
 	E("Malforce Version number");
 	r = chunk_getwordlist(c->control,"Distribution",&c->distributions);
 	E("Missing 'Distribution' field");
-	r = check(filename,c,"Urgency",force);
+	r = check(filename,c,"Urgency");
 	R;
-	r = check(filename,c,"Maintainer",force);
+	r = check(filename,c,"Maintainer");
 	R;
-	r = check(filename,c,"Description",force);
+	r = check(filename,c,"Description");
 	R;
-	r = check(filename,c,"Changes",force);
+	r = check(filename,c,"Changes");
 	R;
 	r = chunk_getextralinelist(c->control,"Files",&filelines);
 	E("Missing 'Files' field");
@@ -420,7 +425,6 @@ static retvalue changes_read(const char *filename,/*@out@*/struct changes **chan
 	*changes = c;
 	return RET_OK;
 #undef E
-#undef C
 #undef R
 }
 
@@ -438,20 +442,20 @@ static retvalue changes_fixfields(const struct distribution *distribution,const
 		return RET_ERROR;
 	}
 	
-	while( e ) {
+	while( e != NULL ) {
 		const struct overrideinfo *oinfo = NULL;
 		const char *force = NULL;
-		if( !forcesection || !forcepriority ) {
+		if( forcesection == NULL || forcepriority == NULL ) {
 			oinfo = override_search(
 			FE_BINARY(e->type)?(e->type==fe_UDEB?ao->udeb:ao->deb):ao->dsc,
 					e->name);
 		}
 		
-		if( forcesection ) 
+		if( forcesection != NULL ) 
 			force = forcesection;
 		else
 			force = override_get(oinfo,SECTION_FIELDNAME);
-		if( force ) {
+		if( force != NULL ) {
 			free(e->section);
 			e->section = strdup(force);
 			if( e->section == NULL )
@@ -469,11 +473,11 @@ static retvalue changes_fixfields(const struct distribution *distribution,const
 			fprintf(stderr,"No section specified for '%s'!\n",filename);
 			return RET_ERROR;
 		}
-		if( forcepriority )
+		if( forcepriority != NULL )
 			force = forcepriority;
 		else
 			force = override_get(oinfo,PRIORITY_FIELDNAME);
-		if( force ) {
+		if( force != NULL ) {
 			free(e->priority);
 			e->priority = strdup(force);
 			if( e->priority == NULL )
@@ -536,6 +540,23 @@ static retvalue changes_fixfields(const struct distribution *distribution,const
 		changes->srcdirectory = calc_sourcedir(changes->srccomponent,changes->source);
 		if( changes->srcdirectory == NULL )
 			return RET_ERROR_OOM;
+	} else if( distribution->trackingoptions.includechanges ) {
+		const char *component = forcecomponent;
+		if( forcecomponent == NULL ) {
+			for( e = changes->files ; e != NULL ; e = e->next ) {
+				if( FE_BINARY(e->type) ){
+					component = e->component;
+					break;
+				}
+			}
+		}
+		if( component == NULL ) {
+			fprintf(stderr,"No component found to place .changes or byhand files in. Aborting.\n");
+			return RET_ERROR;
+		}
+		changes->srcdirectory = calc_sourcedir(component,changes->source);
+		if( changes->srcdirectory == NULL )
+			return RET_ERROR_OOM;
 	}
 
 	return RET_OK;
@@ -545,8 +566,8 @@ static inline retvalue checkforarchitecture(const struct fileentry *e,const char
 	while( e !=NULL && strcmp(e->architecture,architecture) != 0 )
 		e = e->next;
 	if( e == NULL ) {
-		fprintf(stderr,"Architecture-header lists architecture '%s', but no files for this!\n",architecture);
-		return RET_ERROR;
+		if( !IGNORING_(unusedarch,"Architecture-header lists architecture '%s', but no files for this!\n",architecture))
+			return RET_ERROR;
 	}
 	return RET_OK;
 }
@@ -558,12 +579,14 @@ static retvalue changes_check(const char *filename,struct changes *changes,/*@nu
 	bool_t havedsc=FALSE, haveorig=FALSE, havetar=FALSE, havediff=FALSE;
 	
 	/* First check for each given architecture, if it has files: */
-	if( forcearchitecture ) {
+	if( forcearchitecture != NULL ) {
 		if( !strlist_in(&changes->architectures,forcearchitecture) ){
-			fprintf(stderr,"Architecture-header does not list the"
+			// TODO: check if this is sensible
+			if( !IGNORING_(surprisingarch,
+				     "Architecture-header does not list the"
 				     " architecture '%s' to be forced in!\n",
-					forcearchitecture);
-			return RET_ERROR_MISSING;
+					forcearchitecture))
+				return RET_ERROR_MISSING;
 		}
 		r = checkforarchitecture(changes->files,forcearchitecture);
 		if( RET_WAS_ERROR(r) )
@@ -579,10 +602,11 @@ static retvalue changes_check(const char *filename,struct changes *changes,/*@nu
 	/* Then check for each file, if its architecture is sensible
 	 * and listed. */
 	e = changes->files;
-	while( e ) {
+	while( e != NULL ) {
 		if( !strlist_in(&changes->architectures,e->architecture) ) {
-			fprintf(stderr,"'%s' looks like architecture '%s', but this is not listed in the Architecture-Header!\n",filename,e->architecture);
-			r = RET_ERROR;
+			if( !IGNORING_(surprisingarch,
+			"'%s' looks like architecture '%s', but this is not listed in the Architecture-Header!\n",filename,e->architecture))
+				r = RET_ERROR;
 		}
 		if( e->type == fe_DSC ) {
 			char *calculatedname;
@@ -628,7 +652,7 @@ static retvalue changes_check(const char *filename,struct changes *changes,/*@nu
 		return RET_ERROR;
 	}
 	if( havetar && havediff ) {
-		fprintf(stderr,"I don't know what to do having a .tar.gz not beeing a .orig.tar.gz and a .diff.gz in '%s'!\n",filename);
+		fprintf(stderr,"I don't know what to do having a .tar.gz not being a .orig.tar.gz and a .diff.gz in '%s'!\n",filename);
 		return RET_ERROR;
 	}
 	if( strlist_in(&changes->architectures,"source") && !havedsc ) {
@@ -655,7 +679,7 @@ static retvalue changes_includefiles(filesdb filesdb,const char *filename,struct
 	r = RET_NOTHING;
 
 	e = changes->files;
-	while( e ) {
+	while( e != NULL ) {
 		if( FE_SOURCE(e->type) ) {
 			assert(changes->srcdirectory!=NULL);
 			e->filekey = calc_dirconcat(changes->srcdirectory,e->basename);
@@ -687,7 +711,7 @@ static retvalue changes_includefiles(filesdb filesdb,const char *filename,struct
 	return r;
 }
 
-static retvalue changes_includepkgs(const char *dbdir,references refs,filesdb filesdb,struct distribution *distribution,struct changes *changes,const struct alloverrides *ao,int force,/*@null@*/struct strlist *dereferencedfilekeys, bool_t onlysigned) {
+static retvalue changes_checkpkgs(filesdb filesdb,struct distribution *distribution,struct changes *changes,const struct alloverrides *ao, bool_t onlysigned) {
 	struct fileentry *e;
 	retvalue r;
 	bool_t somethingwasmissed = FALSE;
@@ -695,7 +719,7 @@ static retvalue changes_includepkgs(const char *dbdir,references refs,filesdb fi
 	r = RET_NOTHING;
 
 	e = changes->files;
-	while( e ) {
+	while( e != NULL ) {
 		char *fullfilename;
 		if( e->type != fe_DEB && e->type != fe_DSC && e->type != fe_UDEB) {
 			e = e->next;
@@ -705,37 +729,34 @@ static retvalue changes_includepkgs(const char *dbdir,references refs,filesdb fi
 		if( fullfilename == NULL )
 			return RET_ERROR_OOM;
 		if( e->type == fe_DEB ) {
-			r = deb_add(dbdir,refs,filesdb,
+			r = deb_prepare(&e->pkg.deb,filesdb,
 				e->component,e->architecture,
 				e->section,e->priority,
 				"deb",
 				distribution,fullfilename,
 				e->filekey,e->md5sum,
-				ao->deb,
-				force,D_INPLACE,dereferencedfilekeys);
+				ao->deb,D_INPLACE,FALSE);
 			if( r == RET_NOTHING )
 				somethingwasmissed = TRUE;
 		} else if( e->type == fe_UDEB ) {
-			r = deb_add(dbdir,refs,filesdb,
+			r = deb_prepare(&e->pkg.deb,filesdb,
 				e->component,e->architecture,
 				e->section,e->priority,
 				"udeb",
 				distribution,fullfilename,
 				e->filekey,e->md5sum,
-				ao->udeb,
-				force,D_INPLACE,dereferencedfilekeys);
+				ao->udeb,D_INPLACE,FALSE);
 			if( r == RET_NOTHING )
 				somethingwasmissed = TRUE;
 		} else if( e->type == fe_DSC ) {
 			assert(changes->srccomponent!=NULL);
 			assert(changes->srcdirectory!=NULL);
-			r = dsc_add(dbdir,refs,filesdb,
+			r = dsc_prepare(&e->pkg.dsc,filesdb,
 				changes->srccomponent,e->section,e->priority,
 				distribution,fullfilename,
 				e->filekey,e->basename,
 				changes->srcdirectory,e->md5sum,
-				ao->dsc,
-				force,D_INPLACE,dereferencedfilekeys,onlysigned);
+				ao->dsc,D_INPLACE,onlysigned);
 			if( r == RET_NOTHING )
 				somethingwasmissed = TRUE;
 		}
@@ -751,59 +772,155 @@ static retvalue changes_includepkgs(const char *dbdir,references refs,filesdb fi
 	}
 	return r;
 }
+static retvalue changes_includepkgs(const char *dbdir,references refs,struct distribution *distribution,struct changes *changes,/*@null@*/struct strlist *dereferencedfilekeys, /*@null@*/struct trackingdata *trackingdata) {
+	struct fileentry *e;
+	retvalue r;
+	bool_t somethingwasmissed = FALSE;
+
+	r = RET_NOTHING;
+
+	e = changes->files;
+	while( e != NULL ) {
+		if( e->type != fe_DEB && e->type != fe_DSC && e->type != fe_UDEB) {
+			e = e->next;
+			continue;
+		}
+		if( e->type == fe_DEB ) {
+			r = deb_addprepared(e->pkg.deb,dbdir,refs,
+				e->architecture,"deb",
+				distribution,dereferencedfilekeys,trackingdata);
+			if( r == RET_NOTHING )
+				somethingwasmissed = TRUE;
+		} else if( e->type == fe_UDEB ) {
+			r = deb_addprepared(e->pkg.deb,dbdir,refs,
+				e->architecture,"udeb",
+				distribution,dereferencedfilekeys,trackingdata);
+			if( r == RET_NOTHING )
+				somethingwasmissed = TRUE;
+		} else if( e->type == fe_DSC ) {
+			r = dsc_addprepared(e->pkg.dsc,dbdir,refs,
+				distribution,dereferencedfilekeys,trackingdata);
+			if( r == RET_NOTHING )
+				somethingwasmissed = TRUE;
+		}
+		
+		if( RET_WAS_ERROR(r) )
+			break;
+		e = e->next;
+	}
+
+	if( RET_IS_OK(r) && somethingwasmissed ) {
+		return RET_NOTHING;
+	}
+	return r;
+}
 
 /* insert the given .changes into the mirror in the <distribution>
  * if forcecomponent, forcesection or forcepriority is NULL
  * get it from the files or try to guess it. */
-retvalue changes_add(const char *dbdir,references refs,filesdb filesdb,const char *packagetypeonly,const char *forcecomponent,const char *forcearchitecture,const char *forcesection,const char *forcepriority,struct distribution *distribution,const struct alloverrides *ao,const char *changesfilename,int force,int delete,struct strlist *dereferencedfilekeys,bool_t onlysigned) {
-	retvalue r;
+retvalue changes_add(const char *dbdir,trackingdb const tracks,references refs,filesdb filesdb,const char *packagetypeonly,const char *forcecomponent,const char *forcearchitecture,const char *forcesection,const char *forcepriority,struct distribution *distribution,const struct alloverrides *ao,const char *changesfilename,int delete,struct strlist *dereferencedfilekeys,bool_t onlysigned) {
+	retvalue result,r;
 	struct changes *changes;
+	struct trackingdata trackingdata;
 
-	r = changes_read(changesfilename,&changes,packagetypeonly,forcearchitecture,force,onlysigned);
+	r = changes_read(changesfilename,&changes,packagetypeonly,forcearchitecture,onlysigned);
 	if( RET_WAS_ERROR(r) )
 		return r;
-//	if( changes->distributions.count != 1 ) {
-//		fprintf(stderr,"There is not exactly one distribution given!\n");
-//		changes_free(changes);
-//		return RET_ERROR;
-//	}
+
 	if( (distribution->suite == NULL || 
 		!strlist_in(&changes->distributions,distribution->suite)) &&
 	    !strlist_in(&changes->distributions,distribution->codename) ) {
-		fprintf(stderr,"Warning: .changes put in a distribution not listed within it!\n");
+		if( !IGNORING("Ignoring","To ignore",wrongdistribution,".changes put in a distribution not listed within it!\n") )
+			return RET_ERROR;
 	}
+
 	/* look for component, section and priority to be correct or guess them*/
 	r = changes_fixfields(distribution,changesfilename,changes,forcecomponent,forcesection,forcepriority,ao);
-	if( RET_WAS_ERROR(r) ) {
-		changes_free(changes);
-		return r;
-	}
+
 	/* do some tests if values are sensible */
-	r = changes_check(changesfilename,changes,forcearchitecture);
-	if( RET_WAS_ERROR(r) ) {
-		changes_free(changes);
-		return r;
-	}
-	
+	if( !RET_WAS_ERROR(r) )
+		r = changes_check(changesfilename,changes,forcearchitecture);
+
 	/* add files in the pool */
 	//TODO: D_DELETE would fail here, what to do?
-	r = changes_includefiles(filesdb,changesfilename,changes,delete);
+	if( !RET_WAS_ERROR(r) )
+		r = changes_includefiles(filesdb,changesfilename,changes,delete);
+
+	if( !RET_WAS_ERROR(r) )
+		r = changes_checkpkgs(filesdb,distribution,changes,ao,onlysigned);
+
+
 	if( RET_WAS_ERROR(r) ) {
 		changes_free(changes);
 		return r;
 	}
 
+	if( tracks != NULL ) {
+		r = trackingdata_summon(tracks,changes->source,changes->version,&trackingdata);
+		if( RET_WAS_ERROR(r) ) {
+			changes_free(changes);
+			return r;
+		}
+		if( distribution->trackingoptions.includechanges ) {
+			const char *basename;
+			assert( changes->srcdirectory != NULL );
+
+			basename = dirs_basename(changesfilename);
+			changes->changesfilekey = 
+				calc_dirconcat(changes->srcdirectory,basename);
+			if( changes->changesfilekey == NULL ) {
+				changes_free(changes);
+				trackingdata_done(&trackingdata);
+				return RET_ERROR_OOM;
+			}
+			/* always D_COPY, and only delete it afterwards... */
+			r = files_include(filesdb,changesfilename,
+					changes->changesfilekey,
+					NULL,NULL,D_COPY);
+			if( RET_WAS_ERROR(r) ) {
+				changes_free(changes);
+				trackingdata_done(&trackingdata);
+				return r;
+			}
+		}
+	}
+
 	/* add the source and binary packages in the given distribution */
-	r = changes_includepkgs(dbdir,refs,filesdb,
-		distribution,changes,ao,force,
-		dereferencedfilekeys, onlysigned);
+	result = changes_includepkgs(dbdir,refs,
+		distribution,changes,dereferencedfilekeys,
+		(tracks!=NULL)?&trackingdata:NULL);
+
 	if( RET_WAS_ERROR(r) ) {
+		if( tracks != NULL ) {
+			trackingdata_done(&trackingdata);
+		}
 		changes_free(changes);
 		return r;
 	}
 
-	if( delete >= D_MOVE ) {
-		if( r == RET_NOTHING && delete < D_DELETE ) {
+	if( tracks != NULL ) {
+		if( changes->changesfilekey != NULL ) {
+			assert( changes->srcdirectory != NULL );
+
+			r = trackedpackage_addfilekey(tracks,trackingdata.pkg,ft_CHANGES,changes->changesfilekey,FALSE,refs);
+			RET_ENDUPDATE(result,r);
+		}
+		if( trackingdata.isnew ) {
+			r = tracking_put(tracks,trackingdata.pkg);
+		} else {
+			r = tracking_replace(tracks,trackingdata.pkg);
+		}
+		trackingdata_done(&trackingdata);
+		RET_ENDUPDATE(result,r);
+		if( RET_WAS_ERROR(result) ) {
+			changes_free(changes);
+			return result;
+		}
+	}
+
+	if( delete >= D_MOVE && changesfilename != NULL ) {
+		if( result == RET_NOTHING && delete < D_DELETE && 
+				changes->changesfilekey == NULL) {
 			if( verbose >= 0 ) {
 				fprintf(stderr,"Not deleting '%s' as no package was added or some package was missed.\n(Use --delete --delete to delete anyway in such cases)\n",changesfilename);
 			}
@@ -816,6 +933,7 @@ retvalue changes_add(const char *dbdir,references refs,filesdb filesdb,const cha
 			}
 		}
 	}
+	(void)changes_free(changes);
 
 	return RET_OK;
 }
