@@ -12,7 +12,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02111-1301  USA
  */
 #include <config.h>
 
@@ -39,6 +39,7 @@
 #include "files.h"
 #include "extractcontrol.h"
 #include "guesscomponent.h"
+#include "tracking.h"
 
 extern int verbose;
 
@@ -91,6 +92,8 @@ struct debpackage {
 	/* things to be set by deb_read: */
 	char *package,*version,*source,*architecture;
 	char *control;
+	/* only used when tracking */
+	/*@null@*/char *sourceversion;
 	/* things that might still be NULL then: */
 	char *section;
 	char *priority;
@@ -103,14 +106,15 @@ struct debpackage {
 	char *md5sum;
 };
 
-static void deb_free(/*@only@*/struct debpackage *pkg) {
-	if( pkg ) {
+void deb_free(/*@only@*/struct debpackage *pkg) {
+	if( pkg != NULL ) {
 		free(pkg->package);free(pkg->version);
-		free(pkg->source);free(pkg->architecture);
+		free(pkg->source);free(pkg->sourceversion);
+		free(pkg->architecture);
 		free(pkg->control);
 		free(pkg->section);free(pkg->component);
 		free(pkg->priority);
-		if( pkg->filekey )
+		if( pkg->filekey != NULL )
 			strlist_done(&pkg->filekeys);
 		free(pkg->md5sum);
 	}
@@ -118,10 +122,11 @@ static void deb_free(/*@only@*/struct debpackage *pkg) {
 }
 
 /* read the data from a .deb, make some checks and extract some data */
-static retvalue deb_read(/*@out@*/struct debpackage **pkg, const char *filename) {
+static retvalue deb_read(/*@out@*/struct debpackage **pkg, const char *filename, bool_t needssourceversion) {
 	retvalue r;
 	struct debpackage *deb;
 
+	//TODO: this smells like code-duplication with binaries.c
 
 	deb = calloc(1,sizeof(struct debpackage));
 
@@ -163,7 +168,10 @@ static retvalue deb_read(/*@out@*/struct debpackage **pkg, const char *filename)
 		return r;
 	}
 	/* can be there, otherwise we also know what it is */
-	r = chunk_getname(deb->control,"Source",&deb->source,TRUE);
+	if( needssourceversion )
+		r = chunk_getnameandversion(deb->control,"Source",&deb->source,&deb->sourceversion);
+	else
+		r = chunk_getname(deb->control,"Source",&deb->source,TRUE);
 	if( RET_IS_OK(r) ) {
 		// As package-check is strict subset of source-check
 		// this has only be done, if it is not also sourcename:
@@ -181,6 +189,21 @@ static retvalue deb_read(/*@out@*/struct debpackage **pkg, const char *filename)
 		deb_free(deb);
 		return r;
 	}
+	if( needssourceversion ) {
+		if( deb->sourceversion == NULL ) {
+			deb->sourceversion = strdup(deb->version);
+			if( deb->sourceversion == NULL )
+				r = RET_ERROR_OOM;
+			else 
+				r = RET_OK;
+
+		} else
+			r = properversion(deb->sourceversion);
+		if( RET_WAS_ERROR(r) ) {
+			deb_free(deb);
+			return r;
+		}
+	}
 
 	r = getvalue_n(deb->control,PRIORITY_FIELDNAME,&deb->priority);
 	if( RET_WAS_ERROR(r) ) {
@@ -214,28 +237,28 @@ static retvalue deb_complete(struct debpackage *pkg,const struct overrideinfo *o
 	assert( pkg->section != NULL && pkg->priority != NULL);
 
 	size = pkg->md5sum;
-	while( !isblank(*size) && *size )
+	while( !xisblank(*size) && *size != '\0' )
 		size++;
 	replace = addfield_newn("MD5sum",pkg->md5sum, size-pkg->md5sum,NULL);
-	if( !replace )
+	if( replace == NULL )
 		return RET_ERROR_OOM;
-	while( *size && isblank(*size) )
+	while( *size != '\0' && xisblank(*size) )
 		size++;
 	replace = addfield_new("Size",size,replace);
-	if( !replace )
+	if( replace == NULL )
 		return RET_ERROR_OOM;
 	replace = addfield_new("Filename",pkg->filekey,replace);
-	if( !replace )
+	if( replace == NULL )
 		return RET_ERROR_OOM;
 	replace = addfield_new(SECTION_FIELDNAME,pkg->section ,replace);
-	if( !replace )
+	if( replace == NULL )
 		return RET_ERROR_OOM;
 	replace = addfield_new(PRIORITY_FIELDNAME,pkg->priority, replace);
-	if( !replace )
+	if( replace == NULL )
 		return RET_ERROR_OOM;
 
 	replace = override_addreplacefields(override,replace);
-	if( !replace )
+	if( replace == NULL )
 		return RET_ERROR_OOM;
 
 	newchunk  = chunk_replacefields(pkg->control,replace,"Description");
@@ -275,33 +298,11 @@ static retvalue deb_calclocations(struct debpackage *pkg,/*@null@*/const char *g
 	return r;
 }
 
-static retvalue deb_copyfiles(filesdb filesdb,struct debpackage *pkg,const char *debfilename, int delete) {
+retvalue deb_prepare(/*@out@*/struct debpackage **deb,filesdb filesdb,const char * const forcecomponent,const char * const forcearchitecture,const char *forcesection,const char *forcepriority,const char * const packagetype,struct distribution *distribution,const char *debfilename,const char * const givenfilekey,const char * const givenmd5sum,const struct overrideinfo *binoverride,int delete,bool_t needsourceversion){
 	retvalue r;
-
-	r = files_include(filesdb,debfilename,pkg->filekey,NULL,&pkg->md5sum,delete);
-	return r;
-}
-
-static retvalue deb_checkfiles(struct debpackage *pkg,const char *md5sum) {
-	/* Not much to do here, as anything should already be done... */
-
-	pkg->md5sum = strdup(md5sum);
-	if( pkg->md5sum == NULL )
-		return RET_ERROR_OOM;
-	return RET_OK;
-}
-
-/* insert the given .deb into the mirror in <component> in the <distribution>
- * putting things with architecture of "all" into <d->architectures> (and also
- * causing error, if it is not one of them otherwise)
- * if component is NULL, guessing it from the section. */
-
-retvalue deb_add(const char *dbdir,references refs,filesdb filesdb,const char *forcecomponent,const char *forcearchitecture,const char *forcesection,const char *forcepriority,const char *packagetype,struct distribution *distribution,const char *debfilename,const char *givenfilekey,const char *givenmd5sum,const struct overrideinfo *binoverride,int force,int delete,struct strlist *dereferencedfilekeys){
-	retvalue r,result;
 	struct debpackage *pkg;
 	const struct overrideinfo *oinfo;
 	const struct strlist *components;
-	int i;
 
 	assert( (givenmd5sum!=NULL && givenfilekey!=NULL ) ||
 		(givenmd5sum==NULL && givenfilekey==NULL ) );
@@ -310,7 +311,7 @@ retvalue deb_add(const char *dbdir,references refs,filesdb filesdb,const char *f
 
 	/* First taking a closer look to the file: */
 
-	r = deb_read(&pkg,debfilename);
+	r = deb_read(&pkg,debfilename,needsourceversion);
 	if( RET_WAS_ERROR(r) ) {
 		return r;
 	}
@@ -323,7 +324,7 @@ retvalue deb_add(const char *dbdir,references refs,filesdb filesdb,const char *f
 		forcepriority = override_get(oinfo,PRIORITY_FIELDNAME);
 	}
 
-	if( forcesection ) {
+	if( forcesection != NULL ) {
 		free(pkg->section);
 		pkg->section = strdup(forcesection);
 		if( pkg->section == NULL ) {
@@ -331,7 +332,7 @@ retvalue deb_add(const char *dbdir,references refs,filesdb filesdb,const char *f
 			return RET_ERROR_OOM;
 		}
 	}
-	if( forcepriority ) {
+	if( forcepriority != NULL ) {
 		free(pkg->priority);
 		pkg->priority = strdup(forcepriority);
 		if( pkg->priority == NULL ) {
@@ -370,15 +371,13 @@ retvalue deb_add(const char *dbdir,references refs,filesdb filesdb,const char *f
 	
 	/* some sanity checks: */
 
-	if( forcearchitecture != NULL && strcmp(forcearchitecture,"all") == 0 )
-		forcearchitecture = NULL;
-
-	if( forcearchitecture != NULL && 
+	if( forcearchitecture != NULL && strcmp(forcearchitecture,"all") != 0 &&
 			strcmp(pkg->architecture,forcearchitecture) != 0 &&
 			strcmp(pkg->architecture,"all") != 0 ) {
 		fprintf(stderr,"Cannot checking in '%s' into architecture '%s', as it is '%s'!",
 				debfilename,forcearchitecture,pkg->architecture);
 		deb_free(pkg);
+/* TODO: this should be moved upwards...
 		if( delete >= D_DELETE ) {
 			if( verbose >= 0 )
 				fprintf(stderr,"Deleting '%s' as requested!\n",debfilename);
@@ -386,6 +385,7 @@ retvalue deb_add(const char *dbdir,references refs,filesdb filesdb,const char *f
 				fprintf(stderr,"Error deleting '%s': %m\n",debfilename);
 			}
 		}
+*/
 		return RET_ERROR;
 	} else if( strcmp(pkg->architecture,"all") != 0 &&
 	    !strlist_in( &distribution->architectures, pkg->architecture )) {
@@ -393,10 +393,8 @@ retvalue deb_add(const char *dbdir,references refs,filesdb filesdb,const char *f
 				debfilename,pkg->architecture);
 		(void)strlist_fprint(stderr,&distribution->architectures);
 		(void)fputs("'\n",stderr);
-		if( force <= 0 ) {
-			deb_free(pkg);
-			return RET_ERROR;
-		}
+		deb_free(pkg);
+		return RET_ERROR;
 	}
 	if( !strlist_in(components,pkg->component) ) {
 		fprintf(stderr,"While checking in '%s': Would put in component '%s', but that is not available!\n",debfilename,pkg->component);
@@ -409,25 +407,40 @@ retvalue deb_add(const char *dbdir,references refs,filesdb filesdb,const char *f
 		deb_free(pkg);
 		return r;
 	}
-
 	/* then looking if we already have this, or copy it in */
+	if( givenfilekey != NULL ) {
+		assert(givenmd5sum != NULL);
 
-	if( givenfilekey != NULL && givenmd5sum != NULL ) {
-		assert( delete == D_INPLACE );
-		r = deb_checkfiles(pkg,givenmd5sum);
-	} else
-		r = deb_copyfiles(filesdb,pkg,debfilename,delete);
-	if( RET_WAS_ERROR(r) ) {
-		deb_free(pkg);
-		return r;
-	} 
+		pkg->md5sum = strdup(givenmd5sum);
+		if( pkg->md5sum == NULL ) {
+			deb_free(pkg);
+			return RET_ERROR_OOM;
+		} 
+	} else {
+		assert(givenmd5sum == NULL);
+		r = files_include(filesdb,debfilename,pkg->filekey,NULL,&pkg->md5sum,delete);
+		if( RET_WAS_ERROR(r) ) {
+			deb_free(pkg);
+			return r;
+		}
 
+	}
+	assert( pkg->md5sum != NULL );
+	/* Prepare everything that can be prepared beforehand */
 	r = deb_complete(pkg,oinfo);
 	if( RET_WAS_ERROR(r) ) {
 		deb_free(pkg);
 		return r;
 	} 
+	*deb = pkg;
+	return RET_OK;
+}
+
 	
+retvalue deb_addprepared(const struct debpackage *pkg, const char *dbdir,references refs,const char *forcearchitecture,const char *packagetype,struct distribution *distribution,struct strlist *dereferencedfilekeys,struct trackingdata *trackingdata){
+	retvalue r,result;
+	int i;
+
 	/* finaly put it into one or more architectures of the distribution */
 
 	result = RET_NOTHING;
@@ -437,17 +450,17 @@ retvalue deb_add(const char *dbdir,references refs,filesdb filesdb,const char *f
 		r = target_initpackagesdb(t,dbdir);
 		if( !RET_WAS_ERROR(r) ) {
 			retvalue r2;
-			r = target_addpackage(t,refs,pkg->package,pkg->version,pkg->control,&pkg->filekeys,force,FALSE,dereferencedfilekeys);
+			r = target_addpackage(t,refs,pkg->package,pkg->version,pkg->control,&pkg->filekeys,FALSE,dereferencedfilekeys,trackingdata,ft_ARCH_BINARY);
 			r2 = target_closepackagesdb(t);
 			RET_ENDUPDATE(r,r2);
 		}
 		RET_UPDATE(result,r);
-	} else if( forcearchitecture ) {
+	} else if( forcearchitecture != NULL && strcmp(forcearchitecture,"all") != 0 ) {
 		struct target *t = distribution_getpart(distribution,pkg->component,forcearchitecture,packagetype);
 		r = target_initpackagesdb(t,dbdir);
 		if( !RET_WAS_ERROR(r) ) {
 			retvalue r2;
-			r = target_addpackage(t,refs,pkg->package,pkg->version,pkg->control,&pkg->filekeys,force,FALSE,dereferencedfilekeys);
+			r = target_addpackage(t,refs,pkg->package,pkg->version,pkg->control,&pkg->filekeys,FALSE,dereferencedfilekeys,trackingdata,ft_ALL_BINARY);
 			r2 = target_closepackagesdb(t);
 			RET_ENDUPDATE(r,r2);
 		}
@@ -460,14 +473,56 @@ retvalue deb_add(const char *dbdir,references refs,filesdb filesdb,const char *f
 		r = target_initpackagesdb(t,dbdir);
 		if( !RET_WAS_ERROR(r) ) {
 			retvalue r2;
-			r = target_addpackage(t,refs,pkg->package,pkg->version,pkg->control,&pkg->filekeys,force,FALSE,dereferencedfilekeys);
+			r = target_addpackage(t,refs,pkg->package,pkg->version,pkg->control,&pkg->filekeys,FALSE,dereferencedfilekeys,trackingdata,ft_ALL_BINARY);
 			r2 = target_closepackagesdb(t);
 			RET_ENDUPDATE(r,r2);
 		}
 		RET_UPDATE(result,r);
 	}
 
-	deb_free(pkg);
+//	deb_free(pkg);
 
 	return result;
 }
+
+/* insert the given .deb into the mirror in <component> in the <distribution>
+ * putting things with architecture of "all" into <d->architectures> (and also
+ * causing error, if it is not one of them otherwise)
+ * if component is NULL, guessing it from the section. */
+retvalue deb_add(const char *dbdir,references refs,filesdb filesdb,const char *forcecomponent,const char *forcearchitecture,const char *forcesection,const char *forcepriority,const char *packagetype,struct distribution *distribution,const char *debfilename,const char *givenfilekey,const char *givenmd5sum,const struct overrideinfo *binoverride,int delete,struct strlist *dereferencedfilekeys,/*@null@*/trackingdb tracks){
+	struct debpackage *pkg;
+	retvalue r;
+	struct trackingdata trackingdata;
+
+	if( givenfilekey != NULL && givenmd5sum != NULL ) {
+		assert( delete == D_INPLACE );
+	}
+	r = deb_prepare(&pkg,filesdb,forcecomponent,forcearchitecture,forcesection,forcepriority,packagetype,distribution,debfilename,givenfilekey,givenmd5sum,binoverride,delete,tracks!=NULL);
+	if( RET_WAS_ERROR(r) )
+		return r;
+
+	if( tracks != NULL ) {
+		assert(pkg->sourceversion != NULL);
+		r = trackingdata_summon(tracks,pkg->source,pkg->sourceversion,&trackingdata);
+		if( RET_WAS_ERROR(r) ) {
+			deb_free(pkg);
+			return r;
+		}
+	}
+
+	r = deb_addprepared(pkg,dbdir,refs,forcearchitecture,packagetype,distribution,dereferencedfilekeys,(tracks!=NULL)?&trackingdata:NULL);
+	deb_free(pkg);
+
+	if( tracks != NULL ) {
+		retvalue r2;
+		if( trackingdata.isnew ) {
+			r2 = tracking_put(tracks,trackingdata.pkg);
+		} else {
+			r2 = tracking_replace(tracks,trackingdata.pkg);
+		}
+		trackingdata_done(&trackingdata);
+		RET_ENDUPDATE(r,r2);
+	}
+	
+	return r;
+}
