@@ -12,7 +12,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02111-1301  USA
  */
 #include <config.h>
 
@@ -102,7 +102,7 @@ retvalue downloadcache_add(struct downloadcache *cache,filesdb filesdb,struct ap
 	char *fullfilename;
 	retvalue r;
 
-	assert(filesdb && cache && method);
+	assert( filesdb != NULL && cache != NULL && method != NULL );
 	r = files_expect(filesdb,filekey,md5sum);
 	if( r != RET_NOTHING )
 		return r;
