@@ -12,7 +12,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02111-1301  USA
  */
 #include <config.h>
 
@@ -48,7 +48,6 @@ retvalue extractcontrol(char **control,const char *debfile) {
 	int ret;
 	pid_t ar,tar,pid;
 	int status;
-	gzFile f;
 	char *controlchunk;
 
 	retvalue result,r;
@@ -112,9 +111,10 @@ retvalue extractcontrol(char **control,const char *debfile) {
 
 	/* read data: */
 	if( RET_IS_OK(result) ) {
+		gzFile f;
 		//TODO: making a gzdopen here is kinda stupid...
 		f = gzdopen(pipe2[0],"r");
-		if( !f ) {
+		if( f == NULL ) {
 			fprintf(stderr,"Error opening gzip-stream for pipe!\n");
 			RET_UPDATE(result,RET_ERROR);
 		} else {
@@ -128,7 +128,7 @@ retvalue extractcontrol(char **control,const char *debfile) {
 		}
 	}
 	
-	/* avoid beeing a memory leak */
+	/* avoid being a memory leak */
 	if( !(RET_IS_OK(result)) )
 		controlchunk = NULL;
 
@@ -140,13 +140,15 @@ retvalue extractcontrol(char **control,const char *debfile) {
 		} else {
 			if( pid == ar ) {
 				ar = -1;
-				if( !WIFEXITED(status) || WEXITSTATUS(status)) {
+				if( !WIFEXITED(status) || 
+						WEXITSTATUS(status) != 0) {
 					fprintf(stderr,"Error from ar: %d\n",WEXITSTATUS(status));
 					result = RET_ERROR;
 				}
 			} else if( pid == tar ) {
 				tar = -1;
-				if( !WIFEXITED(status) || WEXITSTATUS(status)) {
+				if( !WIFEXITED(status) || 
+						WEXITSTATUS(status) != 0 ) {
 					fprintf(stderr,"Error from tar: %d\n",WEXITSTATUS(status));
 					result = RET_ERROR;
 				}
