@@ -7,7 +7,6 @@
 
 #define VALID_IGNORES \
 	IGN(ignore) \
-	IGN(invalid_distribution) \
 	IGN(forbiddenchar) \
 	IGN(8bit) \
 	IGN(emptyfilenamepart) \
@@ -15,7 +14,13 @@
 	IGN(spaceonlyline) \
 	IGN(malformedchunk) \
 	IGN(shortkeyid) \
-	IGN(unknownfield)
+	IGN(unknownfield) \
+	IGN(wrongdistribution) \
+	IGN(missingfield) \
+	IGN(brokenold) \
+	IGN(brokenversioncmp) \
+	IGN(unusedarch) \
+	IGN(surprisingarch)
 
 
 enum ignore { 
@@ -29,8 +34,8 @@ enum ignore {
 extern int ignored[IGN_COUNT];
 extern bool_t ignore[IGN_COUNT];
 
-#define IGNORING(ignoring,toignore,what,msg_fmt, ...) \
-	({ 	fprintf(stderr, msg_fmt , ## __VA_ARGS__); \
+#define IGNORING(ignoring,toignore,what, ...) \
+	({ 	fprintf(stderr, __VA_ARGS__); \
 		ignored[IGN_ ## what] ++; \
 		if( ignore[IGN_ ## what] ) { \
 			fputs(ignoring " as --ignore=" #what " given.\n",stderr); \
@@ -39,6 +44,7 @@ extern bool_t ignore[IGN_COUNT];
 		} \
 		ignore[IGN_ ## what]; \
 	})
+#define IGNORING_(what, ...) IGNORING("Ignoring","To ignore",what, __VA_ARGS__ )
 #define IGNORABLE(what) ignore[IGN_ ## what]
 
 #define RETURN_IF_ERROR_UNLESS_IGNORED(r,what,msg_fmt, ...) \
@@ -47,6 +53,6 @@ extern bool_t ignore[IGN_COUNT];
 
 void init_ignores(void);
 
-retvalue add_ignore(const char *given);
+retvalue set_ignore(const char *given,bool_t newvalue, enum config_option_owner newowner);
 
 #endif
