@@ -12,7 +12,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02111-1301  USA
  */
 #include <config.h>
 
@@ -52,10 +52,12 @@
 #include "checkin.h"
 #include "downloadcache.h"
 #include "terms.h"
+#include "tracking.h"
+#include "optionsfile.h"
 
 
 #ifndef STD_BASE_DIR
-#define STD_BASE_DIR "/var/spool/reprepro"
+#define STD_BASE_DIR "."
 #endif
 #ifndef STD_METHOD_DIR
 #define STD_METHOD_DIR "/usr/lib/apt/methods"
@@ -82,9 +84,31 @@ static bool_t	nothingiserror = FALSE;
 static bool_t	nolistsdownload = FALSE;
 static bool_t	keepunreferenced = FALSE;
 static bool_t	keepunneededlists = FALSE;
+static bool_t	keepdirectories = FALSE;
 static bool_t	onlyacceptsigned = FALSE;
+static bool_t	askforpassphrase = FALSE;
+static bool_t	skipold = TRUE;
 int		verbose = 0;
 
+/* define for each config value an owner, and only higher owners are allowed
+ * to change something owned by lower owners. */
+enum config_option_owner config_state,
+#define O(x) owner_ ## x = CONFIG_OWNER_DEFAULT
+O(mirrordir), O(distdir), O(dbdir), O(listdir), O(confdir), O(overridedir), O(methoddir), O(section), O(priority), O(component), O(architecture), O(packagetype), O(nothingiserror), O(nolistsdownload), O(keepunreferenced), O(keepunneededlists), O(keepdirectories), O(onlyacceptsigned),O(askforpassphrase), O(skipold);
+#undef O
+	
+#define CONFIGSET(variable,value) if(owner_ ## variable <= config_state) { \
+					owner_ ## variable = config_state; \
+					variable = value; }
+#define CONFIGDUP(variable,value) if(owner_ ## variable <= config_state) { \
+					owner_ ## variable = config_state; \
+					free(variable); \
+					variable = strdup(value); \
+					if( variable == NULL ) { \
+						fputs("Out of Memory!",stderr); \
+						exit(EXIT_FAILURE); \
+					} }
+
 static inline retvalue removeunreferencedfiles(references refs,filesdb files,struct strlist *dereferencedfilekeys) {
 	int i;
 	retvalue result,r;
@@ -95,7 +119,7 @@ static inline retvalue removeunreferencedfiles(references refs,filesdb files,str
 
 		r = references_isused(refs,filekey);
 		if( r == RET_NOTHING ) {
-			r = files_deleteandremove(files,filekey);
+			r = files_deleteandremove(files,filekey,!keepdirectories);
 		}
 		RET_UPDATE(result,r);
 	}
@@ -195,13 +219,13 @@ ACTION_U_F(addmd5sums) {
 	
 	while( fgets(buffer,1999,stdin) != NULL ) {
 		c = strchr(buffer,'\n');
-		if( ! c ) {
+		if( c == NULL ) {
 			fprintf(stderr,"Line too long\n");
 			return RET_ERROR;
 		}
 		*c = '\0';
 		m = strchr(buffer,' ');
-		if( ! m ) {
+		if( m == NULL ) {
 			fprintf(stderr,"Malformed line\n");
 			return RET_ERROR;
 		}
@@ -271,7 +295,7 @@ static retvalue deleteifunreferenced(void *data,const char *filekey,UNUSED(const
 
 	r = references_isused(dist->references,filekey);
 	if( r == RET_NOTHING ) {
-		r = files_deleteandremove(dist->filesdb,filekey);
+		r = files_deleteandremove(dist->filesdb,filekey,!keepdirectories);
 		return r;
 	} else if( RET_IS_OK(r) ) {
 		return RET_NOTHING;
@@ -309,7 +333,7 @@ ACTION_R(addreference) {
 }
 
 
-struct remove_args {/*@temp@*/references refs; int count; /*@temp@*/ const char * const *names; bool_t *gotremoved; int todo;/*@temp@*/struct strlist *removedfiles;};
+struct remove_args {/*@temp@*/references refs; int count; /*@temp@*/ const char * const *names; bool_t *gotremoved; int todo;/*@temp@*/struct strlist *removedfiles;/*@temp@*/struct trackingdata *trackingdata;};
 
 static retvalue remove_from_target(/*@temp@*/void *data, struct target *target) {
 	retvalue result,r;
@@ -323,7 +347,7 @@ static retvalue remove_from_target(/*@temp@*/void *data, struct target *target)
 
 	result = RET_NOTHING;
 	for( i = 0 ; i < d->count ; i++ ){
-		r = target_removepackage(target,d->refs,d->names[i],d->removedfiles);
+		r = target_removepackage(target,d->refs,d->names[i],d->removedfiles,d->trackingdata);
 		if( RET_IS_OK(r) ) {
 			if( ! d->gotremoved[i] )
 				d->todo--;
@@ -340,6 +364,8 @@ ACTION_D_U(remove) {
 	retvalue result,r;
 	struct distribution *distribution;
 	struct remove_args d;
+	trackingdb tracks;
+	struct trackingdata trackingdata;
 
 	if( argc < 3  ) {
 		fprintf(stderr,"reprepro [-C <component>] [-A <architecture>] [-T <type>] remove <codename> <package-names>\n");
@@ -351,6 +377,23 @@ ACTION_D_U(remove) {
 		return r;
 	}
 
+	if( distribution->tracking != dt_NONE ) {
+		r = tracking_initialize(&tracks,dbdir,distribution);
+		if( RET_WAS_ERROR(r) ) {
+			(void)distribution_free(distribution);
+			return r;
+		}
+		r = trackingdata_new(tracks,&trackingdata);
+		if( RET_WAS_ERROR(r) ) {
+			(void)distribution_free(distribution);
+			(void)tracking_done(tracks);
+			return r;
+		}
+		d.trackingdata = &trackingdata;
+	} else {
+		d.trackingdata = NULL;
+	}
+
 	d.count = argc-2;
 	d.names = argv+2;
 	d.todo = d.count;
@@ -365,7 +408,12 @@ ACTION_D_U(remove) {
 	d.removedfiles = NULL;
 
 	if( d.todo < d.count ) {
-		r = distribution_export(distribution,confdir,dbdir,distdir,force,TRUE);
+		r = distribution_export(distribution,confdir,dbdir,distdir,TRUE);
+		RET_ENDUPDATE(result,r);
+	}
+	if( d.trackingdata != NULL ) {
+		trackingdata_done(d.trackingdata);
+		r = tracking_done(tracks);
 		RET_ENDUPDATE(result,r);
 	}
 	r = distribution_free(distribution);
@@ -495,6 +543,7 @@ ACTION_N(listfilter) {
 	}
 
 	result = distribution_foreach_part(distribution,component,architecture,packagetype,listfilter_in_target,condition,force);
+	term_free(condition);
 	r = distribution_free(distribution);
 	RET_ENDUPDATE(result,r);
 	return result;
@@ -513,9 +562,9 @@ ACTION_F(detect) {
 		}
 
 	} else
-		while( fgets(buffer,4999,stdin) ) {
+		while( fgets(buffer,4999,stdin) != NULL ) {
 			nl = strchr(buffer,'\n');
-			if( !nl ) {
+			if( nl == NULL ) {
 				return RET_ERROR;
 			}
 			*nl = '\0';
@@ -538,9 +587,9 @@ ACTION_F(forget) {
 		}
 
 	} else
-		while( fgets(buffer,4999,stdin) ) {
+		while( fgets(buffer,4999,stdin) != NULL ) {
 			nl = strchr(buffer,'\n');
-			if( !nl ) {
+			if( nl == NULL ) {
 				return RET_ERROR;
 			}
 			*nl = '\0';
@@ -603,7 +652,7 @@ ACTION_N(export) {
 			fprintf(stderr,"Exporting %s...\n",d->codename);
 		}
 
-		r = distribution_export(d,confdir,dbdir,distdir,force,FALSE);
+		r = distribution_export(d,confdir,dbdir,distdir,FALSE);
 		RET_UPDATE(result,r);
 		if( RET_WAS_ERROR(r) && force<= 0 )
 			return r;
@@ -643,6 +692,7 @@ ACTION_D(update) {
 		RET_ENDUPDATE(result,r);
 		return result;
 	}
+	assert( RET_IS_OK(result) );
 
 	result = updates_calcindices(listdir,patterns,distributions,&u_distributions);
 	if( RET_WAS_ERROR(result) ) {
@@ -651,12 +701,13 @@ ACTION_D(update) {
 		RET_ENDUPDATE(result,r);
 		return result;
 	}
+	assert( RET_IS_OK(result) );
 
 	if( !keepunneededlists ) {
 		result = updates_clearlists(listdir,u_distributions);
 	}
 	if( !RET_WAS_ERROR(result) )
-		result = updates_update(dbdir,methoddir,filesdb,references,u_distributions,force,nolistsdownload,dereferenced);
+		result = updates_update(dbdir,methoddir,filesdb,references,u_distributions,force,nolistsdownload,skipold,dereferenced);
 	updates_freeupdatedistributions(u_distributions);
 	updates_freepatterns(patterns);
 
@@ -664,7 +715,7 @@ ACTION_D(update) {
 	if( doexport && verbose >= 0 )
 		fprintf(stderr,"Exporting indices...\n");
 	if( doexport )
-		r = distribution_exportandfreelist(distributions,confdir,dbdir,distdir,force);
+		r = distribution_exportandfreelist(distributions,confdir,dbdir,distdir);
 	else
 		r = distribution_freelist(distributions);
 	RET_ENDUPDATE(result,r);
@@ -713,7 +764,7 @@ ACTION_D(iteratedupdate) {
 		result = updates_clearlists(listdir,u_distributions);
 	}
 	if( !RET_WAS_ERROR(result) )
-		result = updates_iteratedupdate(confdir,dbdir,distdir,methoddir,filesdb,references,u_distributions,force,nolistsdownload,dereferenced);
+		result = updates_iteratedupdate(confdir,dbdir,distdir,methoddir,filesdb,references,u_distributions,force,nolistsdownload,skipold,dereferenced);
 	updates_freeupdatedistributions(u_distributions);
 	updates_freepatterns(patterns);
 
@@ -721,7 +772,7 @@ ACTION_D(iteratedupdate) {
 	if( doexport && verbose >= 0 )
 		fprintf(stderr,"Exporting indices...\n");
 	if( doexport )
-		r = distribution_exportandfreelist(distributions,confdir,dbdir,distdir,force);
+		r = distribution_exportandfreelist(distributions,confdir,dbdir,distdir);
 	else
 		r = distribution_freelist(distributions);
 	RET_ENDUPDATE(result,r);
@@ -765,7 +816,7 @@ ACTION_N(checkupdate) {
 		return result;
 	}
 
-	result = updates_checkupdate(dbdir,methoddir,u_distributions,force,nolistsdownload);
+	result = updates_checkupdate(dbdir,methoddir,u_distributions,force,nolistsdownload,skipold);
 
 	updates_freeupdatedistributions(u_distributions);
 	updates_freepatterns(patterns);
@@ -830,6 +881,180 @@ ACTION_R(rereference) {
 
 	return result;
 }
+/***************************retrack****************************/
+struct data_binsrctrack { /*@temp@*/const struct distribution *distribution; /*@temp@*/references refs; trackingdb tracks;};
+
+static retvalue retrack(void *data,struct target *target) {
+	retvalue result,r;
+	struct data_binsrctrack *d = data;
+
+	result = target_initpackagesdb(target,dbdir);
+	if( !RET_WAS_ERROR(result) ) {
+		result = target_retrack(target,d->tracks,d->refs,force);
+		r = target_closepackagesdb(target);
+		RET_ENDUPDATE(result,r);
+	}
+	return result;
+}
+
+ACTION_R(retrack) {
+	retvalue result,r;
+	struct distribution *distributions,*d;
+
+	if( argc < 1 ) {
+		fprintf(stderr,"reprepro retrack [<distributions>]\n");
+		return RET_ERROR;
+	}
+
+	result = distribution_getmatched(confdir,argc-1,argv+1,&distributions);
+	assert( result != RET_NOTHING );
+	if( RET_WAS_ERROR(result) ) {
+		return result;
+	}
+	result = RET_NOTHING;
+	for( d = distributions ; d != NULL ; d = d->next ) {
+		struct data_binsrctrack dat;
+
+		if( verbose > 0 ) {
+			fprintf(stderr,"Chasing %s...\n",d->codename);
+		}
+		dat.distribution = d;
+		dat.refs = references;
+		r = tracking_initialize(&dat.tracks,dbdir,d);
+		if( RET_WAS_ERROR(r) ) {
+			RET_UPDATE(result,r);
+			if( RET_WAS_ERROR(r) && force <= 0 )
+				break;
+			continue;
+		}
+		r = tracking_clearall(dat.tracks);
+		RET_UPDATE(result,r);
+		r = references_remove(references,d->codename);
+		RET_UPDATE(result,r);
+
+		r = distribution_foreach_part(d,component,architecture,packagetype,
+				retrack,&dat,force);
+		RET_UPDATE(result,r);
+		dat.refs = NULL;
+		dat.distribution = NULL;
+		r = tracking_done(dat.tracks);
+		RET_ENDUPDATE(result,r);
+		if( RET_WAS_ERROR(result) && force <= 0 )
+			break;
+	}
+	r = distribution_freelist(distributions);
+	RET_ENDUPDATE(result,r);
+
+	return result;
+}
+
+ACTION_D_U(removetrack) {
+	retvalue result,r;
+	struct distribution *distribution;
+	trackingdb tracks;
+
+	if( argc != 4 ) {
+		fprintf(stderr,"reprepro removetrack <distribution> <sourcename> <version>\n");
+		return RET_ERROR;
+	}
+	result = distribution_get(&distribution,confdir,argv[1]);
+	assert( result != RET_NOTHING );
+	if( RET_WAS_ERROR(result) )
+		return result;
+	r = tracking_initialize(&tracks,dbdir,distribution);
+	if( RET_WAS_ERROR(r) ) {
+		distribution_free(distribution);
+		return r;
+	}
+
+	result = tracking_remove(tracks,argv[2],argv[3],references,dereferenced);
+
+	r = tracking_done(tracks);
+	RET_ENDUPDATE(result,r);
+	r = distribution_free(distribution);
+	RET_ENDUPDATE(result,r);
+	return result;
+}
+	
+ACTION_R(cleartracks) {
+	retvalue result,r;
+	struct distribution *distributions,*d;
+
+	if( argc < 1 ) {
+		fprintf(stderr,"reprepro cleartracks [<distributions>]\n");
+		return RET_ERROR;
+	}
+
+	result = distribution_getmatched(confdir,argc-1,argv+1,&distributions);
+	assert( result != RET_NOTHING );
+	if( RET_WAS_ERROR(result) ) {
+		return result;
+	}
+	result = RET_NOTHING;
+	for( d = distributions ; d != NULL ; d = d->next ) {
+		trackingdb tracks;
+
+		if( verbose > 0 ) {
+			fprintf(stderr,"Deleting all tracks for %s...\n",d->codename);
+		}
+		r = tracking_initialize(&tracks,dbdir,d);
+		if( RET_WAS_ERROR(r) ) {
+			RET_UPDATE(result,r);
+			if( RET_WAS_ERROR(r) && force <= 0 )
+				break;
+			continue;
+		}
+		r = tracking_clearall(tracks);
+		RET_UPDATE(result,r);
+		r = references_remove(references,d->codename);
+		RET_UPDATE(result,r);
+		r = tracking_done(tracks);
+		RET_ENDUPDATE(result,r);
+		if( RET_WAS_ERROR(result) && force <= 0 )
+			break;
+	}
+	r = distribution_freelist(distributions);
+	RET_ENDUPDATE(result,r);
+
+	return result;
+}
+ACTION_N(dumptracks) {
+	retvalue result,r;
+	struct distribution *distributions,*d;
+
+	if( argc < 1 ) {
+		fprintf(stderr,"reprepro dumptracks [<distributions>]\n");
+		return RET_ERROR;
+	}
+
+	result = distribution_getmatched(confdir,argc-1,argv+1,&distributions);
+	assert( result != RET_NOTHING );
+	if( RET_WAS_ERROR(result) ) {
+		return result;
+	}
+	result = RET_NOTHING;
+	for( d = distributions ; d != NULL ; d = d->next ) {
+		trackingdb tracks;
+
+		r = tracking_initialize(&tracks,dbdir,d);
+		if( RET_WAS_ERROR(r) ) {
+			RET_UPDATE(result,r);
+			if( RET_WAS_ERROR(r) && force <= 0 )
+				break;
+			continue;
+		}
+		r = tracking_printall(tracks);
+		RET_UPDATE(result,r);
+		r = tracking_done(tracks);
+		RET_ENDUPDATE(result,r);
+		if( RET_WAS_ERROR(result) && force <= 0 )
+			break;
+	}
+	r = distribution_freelist(distributions);
+	RET_ENDUPDATE(result,r);
+
+	return result;
+}
 /***********************checking*************************/
 struct data_check { /*@temp@*/const struct distribution *distribution; /*@temp@*/references references; /*@temp@*/filesdb filesdb;};
 
@@ -946,7 +1171,7 @@ ACTION_N(reoverride) {
 			break;
 	}
 	if( RET_IS_OK(result) )
-		r = distribution_exportandfreelist(distributions,confdir,dbdir,distdir,force);
+		r = distribution_exportandfreelist(distributions,confdir,dbdir,distdir);
 	else
 		r = distribution_freelist(distributions);
 	RET_ENDUPDATE(result,r);
@@ -962,8 +1187,9 @@ ACTION_D(includedeb) {
 	struct overrideinfo *override;
 	bool_t isudeb;
 	const char *overridefile;
+	trackingdb tracks;
 
-	if( argc < 3 ) {
+	if( argc != 3 ) {
 		fprintf(stderr,"reprepro [--delete] include[u]deb <distribution> <package>\n");
 		return RET_ERROR;
 	}
@@ -992,7 +1218,6 @@ ACTION_D(includedeb) {
 	result = distribution_get(&distribution,confdir,argv[1]);
 	assert( result != RET_NOTHING );
 	if( RET_WAS_ERROR(result) ) {
-		override_free(override);
 		return result;
 	}
 
@@ -1010,18 +1235,34 @@ ACTION_D(includedeb) {
 	// TODO: same for component? (depending on type?)
 	if( architecture != NULL && !strlist_in(&distribution->architectures,architecture) ){
 		fprintf(stderr,"Cannot force into the architecture '%s' not available in '%s'!\n",architecture,distribution->codename);
+		override_free(override);
 		(void)distribution_free(distribution);
 		return RET_ERROR;
 	}
 
+	if( distribution->tracking != dt_NONE ) {
+		result = tracking_initialize(&tracks,dbdir,distribution);
+		if( RET_WAS_ERROR(result) ) {
+			r = distribution_free(distribution);
+			RET_ENDUPDATE(result,r);
+			override_free(override);
+			return result;
+		}
+	} else {
+		tracks = NULL;
+	}
+
 	result = deb_add(dbdir,references,filesdb,component,architecture,
 			section,priority,isudeb?"udeb":"deb",distribution,argv[2],
-			NULL,NULL,override,force,delete,
-			dereferenced);
+			NULL,NULL,override,delete,
+			dereferenced,tracks);
 
 	override_free(override);
 
-	r = distribution_export(distribution,confdir,dbdir,distdir,force,TRUE);
+	r = tracking_done(tracks);
+	RET_ENDUPDATE(result,r);
+
+	r = distribution_export(distribution,confdir,dbdir,distdir,TRUE);
 	RET_ENDUPDATE(result,r);
 
 	r = distribution_free(distribution);
@@ -1035,8 +1276,9 @@ ACTION_D(includedsc) {
 	retvalue result,r;
 	struct distribution *distribution;
 	struct overrideinfo *srcoverride;
+	trackingdb tracks;
 
-	if( argc < 3 ) {
+	if( argc != 3 ) {
 		fprintf(stderr,"reprepro [--delete] includedsc <distribution> <package>\n");
 		return RET_ERROR;
 	}
@@ -1064,10 +1306,24 @@ ACTION_D(includedsc) {
 		}
 	}
 
-	result = dsc_add(dbdir,references,filesdb,component,section,priority,distribution,argv[2],NULL,NULL,NULL,NULL,srcoverride,force,delete,dereferenced,onlyacceptsigned);
+	if( distribution->tracking != dt_NONE ) {
+		result = tracking_initialize(&tracks,dbdir,distribution);
+		if( RET_WAS_ERROR(result) ) {
+			r = distribution_free(distribution);
+			RET_ENDUPDATE(result,r);
+			override_free(srcoverride);
+			return result;
+		}
+	} else {
+		tracks = NULL;
+	}
+
+	result = dsc_add(dbdir,references,filesdb,component,section,priority,distribution,argv[2],NULL,NULL,NULL,NULL,srcoverride,delete,dereferenced,onlyacceptsigned,tracks);
 	
 	override_free(srcoverride);
-	r = distribution_export(distribution,confdir,dbdir,distdir,force,TRUE);
+	r = tracking_done(tracks);
+	RET_ENDUPDATE(result,r);
+	r = distribution_export(distribution,confdir,dbdir,distdir,TRUE);
 	RET_ENDUPDATE(result,r);
 	r = distribution_free(distribution);
 	RET_ENDUPDATE(result,r);
@@ -1079,8 +1335,9 @@ ACTION_D(include) {
 	retvalue result,r;
 	struct distribution *distribution;
 	struct alloverrides ao;
+	trackingdb tracks;
 
-	if( argc < 3 ) {
+	if( argc != 3 ) {
 		fprintf(stderr,"reprepro [--delete] include <distribution> <.changes-file>\n");
 		return RET_ERROR;
 	}
@@ -1097,11 +1354,25 @@ ACTION_D(include) {
 		return result;
 	}
 
-	result = changes_add(dbdir,references,filesdb,packagetype,component,architecture,section,priority,distribution,&ao,argv[2],force,delete,dereferenced,onlyacceptsigned);
+	if( distribution->tracking != dt_NONE ) {
+		result = tracking_initialize(&tracks,dbdir,distribution);
+		if( RET_WAS_ERROR(result) ) {
+			r = distribution_free(distribution);
+			RET_ENDUPDATE(result,r);
+			override_free(ao.deb);override_free(ao.udeb);override_free(ao.dsc);
+			return result;
+		}
+	} else {
+		tracks = NULL;
+	}
+
+	result = changes_add(dbdir,tracks,references,filesdb,packagetype,component,architecture,section,priority,distribution,&ao,argv[2],delete,dereferenced,onlyacceptsigned);
 
 	override_free(ao.deb);override_free(ao.udeb);override_free(ao.dsc);
-	
-	r = distribution_export(distribution,confdir,dbdir,distdir,force,TRUE);
+
+	r = tracking_done(tracks);
+	RET_ENDUPDATE(result,r);
+	r = distribution_export(distribution,confdir,dbdir,distdir,TRUE);
 	RET_ENDUPDATE(result,r);
 	r = distribution_free(distribution);
 	RET_ENDUPDATE(result,r);
@@ -1109,6 +1380,122 @@ ACTION_D(include) {
 	return result;
 }
 
+/***********************createsymlinks***********************************/
+
+ACTION_N(createsymlinks) {
+	retvalue result,r;
+	struct distribution *distributions,*d,*d2;
+
+	r = dirs_make_recursive(distdir);
+	if( RET_WAS_ERROR(r) )
+		return r;
+
+	result = distribution_getmatched(confdir,argc-1,argv+1,&distributions);
+	assert( result != RET_NOTHING );
+	if( RET_WAS_ERROR(result) ) {
+		return result;
+	}
+	result = RET_NOTHING;
+	for( d = distributions ; d != NULL ; d = d->next ) {
+		char *linkname,*buffer;
+		size_t bufsize;
+		int ret;
+		
+		if( d->suite == NULL )
+			continue;
+		r = RET_NOTHING;
+		for( d2 = distributions ; d2 != NULL ; d2 = d2->next ) {
+			if( d!=d2 && d2->suite!=NULL &&strcmp(d->suite,d2->suite)==0) {
+				fprintf(stderr,
+"Not linking %s->%s due to conflict with %s->%s\n",
+					d->suite,d->codename,
+					d2->suite,d2->codename);
+				r = RET_ERROR;
+			} else if( strcmp(d->suite,d2->codename)==0) {
+				fprintf(stderr,
+"Not linking %s->%s due to conflict with %s\n",
+					d->suite,d->codename,d2->codename);
+				r = RET_ERROR;
+			}
+		}
+		if( RET_WAS_ERROR(r) ) {
+			RET_UPDATE(result,r);
+			continue;
+		}
+
+		linkname = calc_dirconcat(distdir,d->suite);
+		bufsize = strlen(d->codename)+10;
+		buffer = calloc(1,bufsize);
+		if( linkname == NULL || buffer == NULL ) {
+			free(linkname);free(buffer);
+			fputs("Out of Memory!\n",stderr);
+			return RET_ERROR_OOM;
+		}
+
+		ret = readlink(linkname,buffer,bufsize-4);
+		if( ret < 0 && errno == ENOENT ) {
+			ret = symlink(d->codename,linkname);
+			if( ret != 0 ) {
+				r = RET_ERRNO(errno);
+				fprintf(stderr,"Error creating symlink %s->%s: %d=%m\n",linkname,d->codename,errno);
+				RET_UPDATE(result,r);
+			} else {
+				if( verbose > 0 ) {
+					printf("Created %s->%s\n",linkname,d->codename);
+				}
+				RET_UPDATE(result,RET_OK);
+			}
+		} else if( ret >= 0 ) {
+			buffer[ret] = '\0';
+			if( ret >= ((int)bufsize)-4 ) {
+				buffer[bufsize-4]='.';
+				buffer[bufsize-3]='.';
+				buffer[bufsize-2]='.';
+				buffer[bufsize-1]='\0';
+			}
+			if( strcmp(buffer,d->codename) == 0 ) {
+				if( verbose > 2 ) {
+					printf("Already ok: %s->%s\n",linkname,d->codename);
+				}
+				RET_UPDATE(result,RET_OK);
+			} else {
+				if( delete <= 0 ) {
+					fprintf(stderr,"Cannot create %s as already pointing to %s instead of %s,\n use --delete to delete the old link before creating an new one.\n",linkname,buffer,d->codename);
+					RET_UPDATE(result,RET_ERROR);
+				} else {
+					unlink(linkname);
+					ret = symlink(d->codename,linkname);
+					if( ret != 0 ) {
+						r = RET_ERRNO(errno);
+						fprintf(stderr,"Error creating symlink %s->%s: %d=%m\n",linkname,d->codename,errno);
+						RET_UPDATE(result,r);
+					} else {
+						if( verbose > 0 ) {
+							printf("Replaced %s->%s\n",linkname,d->codename);
+						}
+						RET_UPDATE(result,RET_OK);
+					}
+
+				}
+			}
+		} else {
+			r = RET_ERRNO(errno);
+			fprintf(stderr,"Error checking %s, perhaps not a symlink?: %d=%m\n",linkname,errno);
+			RET_UPDATE(result,r);
+		}
+
+		RET_UPDATE(result,r);
+	}
+	r = distribution_freelist(distributions);
+	RET_ENDUPDATE(result,r);
+	return result;
+}
+
+
+/**********************/
+/* lock file handling */
+/**********************/
+
 static retvalue acquirelock(const char *dbdir) {
 	char *lockfile;
 	int fd;
@@ -1200,6 +1587,7 @@ static const struct action {
 	{"remove", 		A_D(remove)},
 	{"list", 		A_N(list)},
 	{"listfilter", 		A_N(listfilter)},
+	{"createsymlinks", 	A_N(createsymlinks)},
 	{"export", 		A_N(export)},
 	{"check", 		A_RF(check)},
 	{"reoverride", 		A_N(reoverride)},
@@ -1208,6 +1596,10 @@ static const struct action {
 	{"dumpreferences", 	A_R(dumpreferences)},
 	{"dumpunreferenced", 	A_RF(dumpunreferenced)},
 	{"deleteunreferenced", 	A_RF(deleteunreferenced)},
+	{"retrack",	 	A_R(retrack)},
+	{"dumptracks",	 	A_N(dumptracks)},
+	{"cleartracks",	 	A_R(cleartracks)},
+	{"removetrack",		A_D(removetrack)},
 	{"update",		A_D(update)},
 	{"iteratedupdate",	A_D(iteratedupdate)},
 	{"checkupdate",		A_N(checkupdate)},
@@ -1310,9 +1702,21 @@ static retvalue callaction(const struct action *action,int argc,const char *argv
 #define LO_DELETE 1
 #define LO_KEEPUNREFERENCED 2
 #define LO_KEEPUNNEEDEDLISTS 3
-#define LO_NOHTINGISERROR 4
+#define LO_NOTHINGISERROR 4
 #define LO_NOLISTDOWNLOAD 5
 #define LO_ONLYACCEPTSIGNED 6
+#define LO_ASKPASSPHRASE 7
+#define LO_KEEPDIRECTORIES 8
+#define LO_SKIPOLD 9
+#define LO_NODELETE 21
+#define LO_NOKEEPUNREFERENCED 22
+#define LO_NOKEEPUNNEEDEDLISTS 23
+#define LO_NONOTHINGISERROR 24
+#define LO_LISTDOWNLOAD 25
+#define LO_NOONLYACCEPTSIGNED 26
+#define LO_NOASKPASSPHRASE 27
+#define LO_NOKEEPDIRECTORIES 28
+#define LO_NOSKIPOLD 29
 #define LO_DISTDIR 10
 #define LO_DBDIR 11
 #define LO_LISTDIR 12
@@ -1320,46 +1724,16 @@ static retvalue callaction(const struct action *action,int argc,const char *argv
 #define LO_CONFDIR 14
 #define LO_METHODDIR 15
 #define LO_VERSION 20
+#define LO_UNIGNORE 30
 static int longoption = 0;
+const char *programname;
 
-int main(int argc,char *argv[]) {
-	static struct option longopts[] = {
-		{"delete", 0, &longoption,LO_DELETE},
-		{"basedir", 1, NULL, 'b'},
-		{"ignore", 1, NULL, 'i'},
-		{"methoddir", 1, &longoption, LO_METHODDIR},
-		{"distdir", 1, &longoption, LO_DISTDIR},
-		{"dbdir", 1, &longoption, LO_DBDIR},
-		{"listdir", 1, &longoption, LO_LISTDIR},
-		{"overridedir", 1, &longoption, LO_OVERRIDEDIR},
-		{"confdir", 1, &longoption, LO_CONFDIR},
-		{"section", 1, NULL, 'S'},
-		{"priority", 1, NULL, 'P'},
-		{"component", 1, NULL, 'C'},
-		{"architecture", 1, NULL, 'A'},
-		{"type", 1, NULL, 'T'},
-		{"help", 0, NULL, 'h'},
-		{"verbose", 0, NULL, 'v'},
-		{"version", 0, &longoption, LO_VERSION},
-		{"nothingiserror", 0, &longoption, LO_NOHTINGISERROR},
-		{"nolistsdownload", 0, &longoption, LO_NOLISTDOWNLOAD},
-		{"keepunreferencedfiles", 0, &longoption, LO_KEEPUNREFERENCED},
-		{"keepunneededlists", 0, &longoption, LO_KEEPUNNEEDEDLISTS},
-		{"onlyacceptsigned", 0, &longoption, LO_ONLYACCEPTSIGNED},
-		{"force", 0, NULL, 'f'},
-		{NULL, 0, NULL, 0}
-	};
-	const struct action *a;
+static void handle_option(int c,const char *optarg) {
 	retvalue r;
-	int c;
 
-	init_ignores();
-
-
-	while( (c = getopt_long(argc,argv,"+fVvhb:P:i:A:C:S:T:",longopts,NULL)) != -1 ) {
-		switch( c ) {
-			case 'h':
-				printf(
+	switch( c ) {
+		case 'h':
+			printf(
 "reprepro - Produce and Manage and Debian package repository\n\n"
 "options:\n"
 " -h, --help:                        Show this help\n"
@@ -1405,145 +1779,256 @@ int main(int argc,char *argv[]) {
 "       List all packages by the given name occuring in the given distribution.\n"
 " listfilter <distribution> <condition>\n"
 "       List all packages in the given distribution matching the condition.\n"
-"\n"
-						);
-				exit(EXIT_SUCCESS);
-			case '\0':
-				switch( longoption ) {
-					case LO_DELETE:
-						delete++;
-						break;
-					case LO_ONLYACCEPTSIGNED:
-						onlyacceptsigned = TRUE;
-						break;
-					case LO_KEEPUNREFERENCED:
-						keepunreferenced=TRUE;
-						break;
-					case LO_KEEPUNNEEDEDLISTS:
-						keepunneededlists=TRUE;
-						break;
-					case LO_NOHTINGISERROR:
-						nothingiserror=TRUE;
-						break;
-					case LO_NOLISTDOWNLOAD:
-						nolistsdownload=TRUE;
-						break;
-					case LO_DISTDIR:
-						free(distdir);
-						distdir = strdup(optarg);
-						break;
-					case LO_DBDIR:
-						free(dbdir);
-						dbdir = strdup(optarg);
-						break;
-					case LO_LISTDIR:
-						free(listdir);
-						listdir = strdup(optarg);
-						break;
-					case LO_OVERRIDEDIR:
-						free(overridedir);
-						overridedir = strdup(optarg);
-						break;
-					case LO_CONFDIR:
-						free(confdir);
-						confdir = strdup(optarg);
-						break;
-					case LO_METHODDIR:
-						free(methoddir);
-						methoddir = strdup(optarg);
-						break;
-					case LO_VERSION:
-						fprintf(stderr,"%s: This is " PACKAGE " version " VERSION "\n",argv[0]);
-						exit(EXIT_SUCCESS);
-					default:
-						fprintf (stderr,"Error parsing arguments!\n");
+"\n");
+			exit(EXIT_SUCCESS);
+		case '\0':
+			switch( longoption ) {
+				case LO_UNIGNORE:
+					r = set_ignore(optarg,FALSE,config_state);
+					if( RET_WAS_ERROR(r) ) {
 						exit(EXIT_FAILURE);
-				}
-				longoption = 0;
-				break;
-			case 'v':
-				verbose++;
-				break;
-			case 'V':
-				verbose+=5;
-				break;
-			case 'f':
-				force++;
-				break;
-			case 'b':
-				free(mirrordir);
-				mirrordir=strdup(optarg);
-				break;
-			case 'i':
-				r = add_ignore(optarg);
-				if( RET_WAS_ERROR(r) ) {
+					}
+					break;
+				case LO_DELETE:
+					delete++;
+					break;
+				case LO_NODELETE:
+					delete--;
+					break;
+				case LO_ONLYACCEPTSIGNED:
+					CONFIGSET(onlyacceptsigned,TRUE);
+					break;
+				case LO_NOONLYACCEPTSIGNED:
+					CONFIGSET(onlyacceptsigned,FALSE);
+					break;
+				case LO_KEEPUNREFERENCED:
+					CONFIGSET(keepunreferenced,TRUE);
+					break;
+				case LO_NOKEEPUNREFERENCED:
+					CONFIGSET(keepunreferenced,FALSE);
+					break;
+				case LO_KEEPUNNEEDEDLISTS:
+					CONFIGSET(keepunneededlists,TRUE);
+					break;
+				case LO_NOKEEPUNNEEDEDLISTS:
+					CONFIGSET(keepunneededlists,FALSE);
+					break;
+				case LO_KEEPDIRECTORIES:
+					CONFIGSET(keepdirectories,TRUE);
+					break;
+				case LO_NOKEEPDIRECTORIES:
+					CONFIGSET(keepdirectories,FALSE);
+					break;
+				case LO_NOTHINGISERROR:
+					CONFIGSET(nothingiserror,TRUE);
+					break;
+				case LO_NONOTHINGISERROR:
+					CONFIGSET(nothingiserror,FALSE);
+					break;
+				case LO_NOLISTDOWNLOAD:
+					CONFIGSET(nolistsdownload,TRUE);
+					break;
+				case LO_LISTDOWNLOAD:
+					CONFIGSET(nolistsdownload,FALSE);
+					break;
+				case LO_ASKPASSPHRASE:
+					CONFIGSET(askforpassphrase,TRUE);
+					break;
+				case LO_NOASKPASSPHRASE:
+					CONFIGSET(askforpassphrase,FALSE);
+					break;
+				case LO_SKIPOLD:
+					CONFIGSET(skipold,TRUE);
+					break;
+				case LO_NOSKIPOLD:
+					CONFIGSET(skipold,FALSE);
+					break;
+				case LO_DISTDIR:
+					CONFIGDUP(distdir,optarg);
+					break;
+				case LO_DBDIR:
+					CONFIGDUP(dbdir,optarg);
+					break;
+				case LO_LISTDIR:
+					CONFIGDUP(listdir,optarg);
+					break;
+				case LO_OVERRIDEDIR:
+					CONFIGDUP(overridedir,optarg);
+					break;
+				case LO_CONFDIR:
+					CONFIGDUP(confdir,optarg);
+					break;
+				case LO_METHODDIR:
+					CONFIGDUP(methoddir,optarg);
+					break;
+				case LO_VERSION:
+					fprintf(stderr,"%s: This is " PACKAGE " version " VERSION "\n",programname);
+					exit(EXIT_SUCCESS);
+				default:
+					fprintf (stderr,"Error parsing arguments!\n");
 					exit(EXIT_FAILURE);
-				}
-				break;
-			case 'C':
-				if( component != NULL && 
-						strcmp(component,optarg) != 0) {
-					fprintf(stderr,"Multiple '-C' are not supported!\n");
-					exit(EXIT_FAILURE);
-				}
-				free(component);
-				component = strdup(optarg);
-				break;
-			case 'A':
-				if( architecture != NULL && 
-						strcmp(architecture,optarg) != 0) {
-					fprintf(stderr,"Multiple '-A's are not supported!\n");
-					exit(EXIT_FAILURE);
-				}
-				free(architecture);
-				architecture = strdup(optarg);
-				break;
-			case 'T':
-				if( packagetype != NULL && 
-						strcmp(packagetype,optarg) != 0) {
-					fprintf(stderr,"Multiple '-T's are not supported!\n");
-					exit(EXIT_FAILURE);
-				}
-				free(packagetype);
-				packagetype = strdup(optarg);
-				break;
-			case 'S':
-				if( section != NULL && 
-						strcmp(section,optarg) != 0) {
-					fprintf(stderr,"Multiple '-S' are not supported!\n");
-					exit(EXIT_FAILURE);
-				}
-				free(section);
-				section = strdup(optarg);
-				break;
-			case 'P':
-				if( priority != NULL && 
-						strcmp(priority,optarg) != 0) {
-					fprintf(stderr,"Multiple '-P's are mpt supported!\n");
-					exit(EXIT_FAILURE);
-				}
-				free(priority);
-				priority = strdup(optarg);
-				break;
-			case '?':
-				/* getopt_long should have already given an error msg */
+			}
+			longoption = 0;
+			break;
+		case 'v':
+			verbose++;
+			break;
+		case 'V':
+			verbose+=5;
+			break;
+		case 'f':
+			force++;
+			break;
+		case 'b':
+			CONFIGDUP(mirrordir,optarg);
+			break;
+		case 'i':
+			r = set_ignore(optarg,TRUE,config_state);
+			if( RET_WAS_ERROR(r) ) {
 				exit(EXIT_FAILURE);
-			default:
-				fprintf (stderr,"Not supported option '-%c'\n", c);
+			}
+			break;
+		case 'C':
+			if( component != NULL && 
+					strcmp(component,optarg) != 0) {
+				fprintf(stderr,"Multiple '-C' are not supported!\n");
 				exit(EXIT_FAILURE);
-		}
+			}
+			CONFIGDUP(component,optarg);
+			break;
+		case 'A':
+			if( architecture != NULL && 
+					strcmp(architecture,optarg) != 0) {
+				fprintf(stderr,"Multiple '-A's are not supported!\n");
+				exit(EXIT_FAILURE);
+			}
+			CONFIGDUP(architecture,optarg);
+			break;
+		case 'T':
+			if( packagetype != NULL && 
+					strcmp(packagetype,optarg) != 0) {
+				fprintf(stderr,"Multiple '-T's are not supported!\n");
+				exit(EXIT_FAILURE);
+			}
+			CONFIGDUP(packagetype,optarg);
+			break;
+		case 'S':
+			if( section != NULL && 
+					strcmp(section,optarg) != 0) {
+				fprintf(stderr,"Multiple '-S' are not supported!\n");
+				exit(EXIT_FAILURE);
+			}
+			CONFIGDUP(section,optarg);
+			break;
+		case 'P':
+			if( priority != NULL && 
+					strcmp(priority,optarg) != 0) {
+				fprintf(stderr,"Multiple '-P's are mpt supported!\n");
+				exit(EXIT_FAILURE);
+			}
+			CONFIGDUP(priority,optarg);
+			break;
+		case '?':
+			/* getopt_long should have already given an error msg */
+			exit(EXIT_FAILURE);
+		default:
+			fprintf (stderr,"Not supported option '-%c'\n", c);
+			exit(EXIT_FAILURE);
+	}
+}
+
+
+int main(int argc,char *argv[]) {
+	static struct option longopts[] = {
+		{"delete", no_argument, &longoption,LO_DELETE},
+		{"nodelete", no_argument, &longoption,LO_NODELETE},
+		{"basedir", required_argument, NULL, 'b'},
+		{"ignore", required_argument, NULL, 'i'},
+		{"unignore", required_argument, &longoption, LO_UNIGNORE},
+		{"noignore", required_argument, &longoption, LO_UNIGNORE},
+		{"methoddir", required_argument, &longoption, LO_METHODDIR},
+		{"distdir", required_argument, &longoption, LO_DISTDIR},
+		{"dbdir", required_argument, &longoption, LO_DBDIR},
+		{"listdir", required_argument, &longoption, LO_LISTDIR},
+		{"overridedir", required_argument, &longoption, LO_OVERRIDEDIR},
+		{"confdir", required_argument, &longoption, LO_CONFDIR},
+		{"section", required_argument, NULL, 'S'},
+		{"priority", required_argument, NULL, 'P'},
+		{"component", required_argument, NULL, 'C'},
+		{"architecture", required_argument, NULL, 'A'},
+		{"type", required_argument, NULL, 'T'},
+		{"help", no_argument, NULL, 'h'},
+		{"verbose", no_argument, NULL, 'v'},
+		{"version", no_argument, &longoption, LO_VERSION},
+		{"nothingiserror", no_argument, &longoption, LO_NOTHINGISERROR},
+		{"nolistsdownload", no_argument, &longoption, LO_NOLISTDOWNLOAD},
+		{"keepunreferencedfiles", no_argument, &longoption, LO_KEEPUNREFERENCED},
+		{"keepunneededlists", no_argument, &longoption, LO_KEEPUNNEEDEDLISTS},
+		{"keepdirectories", no_argument, &longoption, LO_KEEPDIRECTORIES},
+		{"onlyacceptsigned", no_argument, &longoption, LO_ONLYACCEPTSIGNED},
+		{"ask-passphrase", no_argument, &longoption, LO_ASKPASSPHRASE},
+		{"nonothingiserror", no_argument, &longoption, LO_NONOTHINGISERROR},
+		{"nonolistsdownload", no_argument, &longoption, LO_LISTDOWNLOAD},
+		{"listsdownload", no_argument, &longoption, LO_LISTDOWNLOAD},
+		{"nokeepunreferencedfiles", no_argument, &longoption, LO_NOKEEPUNREFERENCED},
+		{"nokeepunneededlists", no_argument, &longoption, LO_NOKEEPUNNEEDEDLISTS},
+		{"nokeepdirectories", no_argument, &longoption, LO_NOKEEPDIRECTORIES},
+		{"noonlyacceptsigned", no_argument, &longoption, LO_NOONLYACCEPTSIGNED},
+		{"noask-passphrase", no_argument, &longoption, LO_NOASKPASSPHRASE},
+		{"skipold", no_argument, &longoption, LO_SKIPOLD},
+		{"noskipold", no_argument, &longoption, LO_NOSKIPOLD},
+		{"nonoskipold", no_argument, &longoption, LO_SKIPOLD},
+		{"force", no_argument, NULL, 'f'},
+		{NULL, 0, NULL, 0}
+	};
+	const struct action *a;
+	retvalue r;
+	int c;
+
+	programname = argv[0];
+
+	init_ignores();
+
+	config_state = CONFIG_OWNER_CMDLINE;
+
+	while( (c = getopt_long(argc,argv,"+fVvhb:P:i:A:C:S:T:",longopts,NULL)) != -1 ) {
+		handle_option(c,optarg);
 	}
 	if( optind >= argc ) {
 		fprintf(stderr,"No action given. (see --help for available options and actions)\n");
 		exit(EXIT_FAILURE);
 	}
+
+	/* only for this CONFIG_OWNER_ENVIRONMENT is a bit stupid,
+	 * but perhaps it gets more... */
+	config_state = CONFIG_OWNER_ENVIRONMENT;
+	if( mirrordir == NULL && getenv("REPREPRO_BASE_DIR") != NULL ) {
+		CONFIGDUP(mirrordir,getenv("REPREPRO_BASE_DIR"));
+	}
+	if( confdir == NULL && getenv("REPREPRO_CONFIG_DIR") != NULL ) {
+		CONFIGDUP(confdir,getenv("REPREPRO_CONFIG_DIR"));
+	}
+
 	if( mirrordir == NULL ) {
 		mirrordir=strdup(STD_BASE_DIR);
-		if( mirrordir == NULL ) {
-			(void)fputs("Out of Memory!\n",stderr);
-			exit(EXIT_FAILURE);
-		}
 	}
+	if( confdir == NULL && mirrordir != NULL )
+		confdir=calc_dirconcat(mirrordir,"conf");
+
+	if( mirrordir == NULL || confdir == NULL ) {
+		(void)fputs("Out of Memory!\n",stderr);
+		exit(EXIT_FAILURE);
+	}
+
+	config_state = CONFIG_OWNER_FILE;
+	optionsfile_parse(confdir,longopts,handle_option);
+
+	/* basedir might have changed, so recalculate */
+	if( owner_confdir == CONFIG_OWNER_DEFAULT ) {
+		free(confdir);
+		confdir=calc_dirconcat(mirrordir,"conf");
+	}
+
 	if( methoddir == NULL )
 		methoddir = strdup(STD_METHOD_DIR);
 	if( distdir == NULL )
@@ -1552,8 +2037,6 @@ int main(int argc,char *argv[]) {
 		dbdir=calc_dirconcat(mirrordir,"db");
 	if( listdir == NULL )
 		listdir=calc_dirconcat(mirrordir,"lists");
-	if( confdir == NULL )
-		confdir=calc_dirconcat(mirrordir,"conf");
 	if( overridedir == NULL )
 		overridedir=calc_dirconcat(mirrordir,"override");
 	if( distdir == NULL || dbdir == NULL || listdir == NULL 
@@ -1562,11 +2045,12 @@ int main(int argc,char *argv[]) {
 		exit(EXIT_FAILURE);
 	}
 	a = all_actions;
-	while( a->name ) {
+	while( a->name != NULL ) {
 		if( strcasecmp(a->name,argv[optind]) == 0 ) {
+			signature_init(askforpassphrase);
 			r = callaction(a,argc-optind,(const char**)argv+optind);
 			/* yeah, freeing all this stuff before exiting is
-			 * stupid, but it keeps valgrind logs easier 
+			 * stupid, but it makes valgrind logs easier 
 			 * readable */
 			signatures_done();
 			free(dbdir);
