@@ -12,7 +12,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02111-1301  USA
  */
 #include <config.h>
 
@@ -158,9 +158,7 @@ retvalue references_check(references refs,const char *referee,const struct strli
 	return ret;
 }
 
-/* add an reference to a file for an identifier. multiple calls
- * will add multiple references to allow source packages to share
- * files over versions. (as first the new is added, then the old removed) */
+/* add an reference to a file for an identifier. multiple calls */
 retvalue references_increment(references refs,const char *needed,const char *neededby) {
 	int dbret;
 	DBT key,data;
@@ -177,7 +175,7 @@ retvalue references_increment(references refs,const char *needed,const char *nee
 	}
 }
 
-/* remove *one* reference for a file from a given reference */
+/* remove reference for a file from a given reference */
 retvalue references_decrement(references refs,const char *needed,const char *neededby) {
 	DBC *cursor;
 	DBT key,data;
@@ -252,7 +250,7 @@ retvalue references_delete(references refs,const char *identifier,
 		if( exclude == NULL || !strlist_in(exclude,filekey) ) {
 			r = references_decrement(refs,filekey,identifier);
 			RET_UPDATE(result,r);
-			if( RET_IS_OK(r) && dereferencedfilekeys ) {
+			if( RET_IS_OK(r) && dereferencedfilekeys != NULL ) {
 				r = strlist_adduniq(dereferencedfilekeys, filekey);
 				RET_UPDATE(result,r);
 			} else
