@@ -7,6 +7,9 @@
 #ifndef REPREPRO_NAMES_H
 #include "names.h"
 #endif
+#ifndef REPREPRO_TRACKINGT_H
+#include "trackingt.h"
+#endif
 #ifndef REPREPRO_PACKAGES_H
 #include "packages.h"
 #endif
@@ -25,6 +28,8 @@ typedef retvalue get_filekeys(struct target *,const char *,/*@out@*/struct strli
 typedef char *get_upstreamindex(struct target *,const char *suite_from,
 		const char *component_from,const char *architecture);
 typedef retvalue do_reoverride(const struct alloverrides *,const char *packagename,const char *controlchunk,/*@out@*/char **newcontrolchunk);
+typedef retvalue do_retrack(struct target *,const char *packagename,const char *controlchunk,trackingdb,references);
+typedef retvalue get_sourceandversion(struct target *,const char *chunk,const char *packagename,char **source,char **version);
 
 struct target {
 	char *codename;
@@ -43,7 +48,9 @@ struct target {
 	get_installdata *getinstalldata;
 	get_filekeys *getfilekeys;
 	get_upstreamindex *getupstreamindex;
+	get_sourceandversion *getsourceandversion;
 	do_reoverride *doreoverride;
+	do_retrack *doretrack;
 	bool_t wasmodified;
 	/* the next one in the list of targets of a distribution */
 	struct target *next;
@@ -57,7 +64,7 @@ retvalue target_initialize_source(const char *codename,const char *component,/*@
 retvalue target_free(struct target *target);
 
 retvalue target_mkdistdir(struct target *target,const char *distdir);
-retvalue target_export(struct target *target,const char *confdir,const char *dbdir,const char *dirofdist,int force,bool_t onlyneeded, struct strlist *releasedfiles );
+retvalue target_export(struct target *target,const char *confdir,const char *dbdir,bool_t onlyneeded, struct release *release);
 
 retvalue target_printmd5sums(const char *dirofdist,const struct target *target,FILE *out,int force);
 
@@ -68,11 +75,12 @@ retvalue target_closepackagesdb(struct target *target);
 
 /* The following calls can only be called if target_initpackagesdb was called before: */
 
-retvalue target_addpackage(struct target *target,references refs,const char *name,const char *version,const char *control,const struct strlist *filekeys,int force,bool_t downgrade,/*@null@*/struct strlist *dereferencedfilekeys);
-retvalue target_removepackage(struct target *target,references refs,const char *name, /*@null@*/struct strlist *dereferencedfilekeys);
+retvalue target_addpackage(struct target *target,references refs,const char *name,const char *version,const char *control,const struct strlist *filekeys,bool_t downgrade,/*@null@*/struct strlist *dereferencedfilekeys,/*@null@*/struct trackingdata *,enum filetype);
+retvalue target_removepackage(struct target *target,references refs,const char *name, /*@null@*/struct strlist *dereferencedfilekeys,struct trackingdata *);
 retvalue target_writeindices(const char *dirofdist,struct target *target,int force,bool_t onlyneeded);
 retvalue target_check(struct target *target,filesdb filesdb,references refsdb,int force);
 retvalue target_rereference(struct target *target,references refs,int force);
+retvalue target_retrack(struct target *target,trackingdb tracks,references refs,int force);
 retvalue target_reoverride(struct target *target,const struct alloverrides *ao);
 
 #endif
