@@ -12,7 +12,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02111-1301  USA
  */
 #include <config.h>
 
@@ -43,6 +43,7 @@
 #include "distribution.h"
 #include "terms.h"
 #include "filterlist.h"
+#include "readrelease.h"
 
 // TODO: what about other signatures? Is hard-coding ".gpg" sensible?
 
@@ -93,6 +94,8 @@ struct update_pattern {
 	char *name;
 	//e.g. "Method: ftp://ftp.uni-freiburg.de/pub/linux/debian"
 	char *method;
+	//e.g. "Fallback: ftp://ftp.debian.org/pub/linux/debian"
+	char *fallback; // can be other server or dir, but must be same method
 	//e.g. "Config: Dir=/"
 	/*@null@*/char *config;
 	//e.g. "Suite: woody" or "Suite: <asterix>/updates" (NULL means "*")
@@ -150,6 +153,8 @@ struct update_target {
 	/*@null@*/struct upgradelist *upgradelist;
 	/* Ignore delete marks (as some lists were missing) */
 	bool_t ignoredelete;
+	/* don't do anything because of --skipold */
+	bool_t nothingnew;
 };
 
 struct update_distribution {
@@ -165,6 +170,7 @@ static void update_pattern_free(/*@only@*/struct update_pattern *update) {
 	free(update->name);
 	free(update->config);
 	free(update->method);
+	free(update->fallback);
 	free(update->suite_from);
 	free(update->verifyrelease);
 	strlist_done(&update->architectures_from);
@@ -180,7 +186,7 @@ static void update_pattern_free(/*@only@*/struct update_pattern *update) {
 }
 
 void updates_freepatterns(struct update_pattern *p) {
-	while( p ) {
+	while( p != NULL ) {
 		struct update_pattern *pattern;
 
 		pattern = p;
@@ -190,7 +196,7 @@ void updates_freepatterns(struct update_pattern *p) {
 }
 
 static void updates_freeorigins(/*@only@*/struct update_origin *o) {
-	while( o ) {
+	while( o != NULL ) {
 		struct update_origin *origin;
 
 		origin = o;
@@ -204,12 +210,12 @@ static void updates_freeorigins(/*@only@*/struct update_origin *o) {
 }
 
 static void updates_freetargets(/*@only@*/struct update_target *t) {
-	while( t ) {
+	while( t != NULL ) {
 		struct update_target *ut;
 
 		ut = t;
 		t = ut->next;
-		while( ut->indices ) {
+		while( ut->indices != NULL ) {
 			struct update_index *ui;
 
 			ui = ut->indices;
@@ -223,7 +229,7 @@ static void updates_freetargets(/*@only@*/struct update_target *t) {
 }
 
 void updates_freeupdatedistributions(struct update_distribution *d) {
-	while( d ) {
+	while( d != NULL ) {
 		struct update_distribution *next;
 
 		next = d->next;
@@ -245,6 +251,7 @@ static inline retvalue newupdatetarget(struct update_target **ts,/*@dependent@*/
 	ut->indices = NULL;
 	ut->upgradelist = NULL;
 	ut->ignoredelete = FALSE;
+	ut->nothingnew = FALSE;
 	*ts = ut;
 	return RET_OK;
 }
@@ -273,7 +280,7 @@ static retvalue splitlist(struct strlist *from,
 
 		item = list->values[i];
 		// TODO: isn't this broken for the !*(dest+1) case ?
-		if( !(seperator = strchr(item,'>')) ) {
+		if( (seperator = strchr(item,'>')) == NULL ) {
 			destination = strdup(item);
 			origin = strdup(item);
 		} else if( seperator == item ) {
@@ -286,7 +293,7 @@ static retvalue splitlist(struct strlist *from,
 			origin = strndup(item,seperator-item);
 			destination = strdup(seperator+1);
 		}
-		if( !origin || ! destination ) {
+		if( origin == NULL || destination == NULL ) {
 			free(origin);free(destination);
 			strlist_done(from);
 			strlist_done(into);
@@ -318,7 +325,7 @@ inline static retvalue parse_pattern(const char *confdir,const char *chunk, stru
 	static const char * const allowedfields[] = {"Name", "Method", 
 "Config", "Suite", "Architectures", "Components", "UDebComponents", 
 "IgnoreRelease", "VerifyRelease", "FilterFormula", "FilterList",
-"ListHook", NULL};
+"ListHook", "Fallback", NULL};
 
 	update = calloc(1,sizeof(struct update_pattern));
 	if( update == NULL )
@@ -348,6 +355,15 @@ inline static retvalue parse_pattern(const char *confdir,const char *chunk, stru
 		return r;
 	}
 
+	/* * Look for alternate server: * */
+	r = chunk_getvalue(chunk,"Fallback",&update->fallback);
+	if( r == RET_NOTHING )
+		update->fallback = NULL;
+	else if( !RET_IS_OK(r) ) {
+		update_pattern_free(update);
+		return r;
+	}
+
 	// TODO: if those are checked anyway, there should be no reason to
 	// research them later...
 	r = chunk_checkfields(chunk,allowedfields,TRUE);
@@ -550,7 +566,7 @@ static retvalue instance_pattern(const char *listdir,
 			updates_freeorigins(update);
 			return RET_ERROR_OOM;
 		}
-		if( pattern->verifyrelease ) {
+		if( pattern->verifyrelease != NULL ) {
 			update->releasegpgfile = calc_downloadedlistfile(listdir,distribution->codename,pattern->name,"Release","gpg","rel");
 			if( update->releasegpgfile == NULL ) {
 				updates_freeorigins(update);
@@ -588,7 +604,7 @@ retvalue updates_getpatterns(const char *confdir,struct update_pattern **pattern
 	retvalue r;
 
 	updatesfile = calc_dirconcat(confdir,"updates");
-	if( !updatesfile ) 
+	if( updatesfile == NULL ) 
 		return RET_ERROR_OOM;
 	data.patterns = &update;
 	data.confdir = confdir;
@@ -596,6 +612,10 @@ retvalue updates_getpatterns(const char *confdir,struct update_pattern **pattern
 	free(updatesfile);
 	if( RET_IS_OK(r) )
 		*patterns = update;
+	else if( r == RET_NOTHING ) {
+		*patterns = NULL;
+		r = RET_OK;
+	}
 	return r;
 }
 
@@ -623,7 +643,7 @@ static retvalue getorigins(const char *listdir,const struct update_pattern *patt
 			continue;
 		}
 
-		for( pattern = patterns ; pattern ; pattern = pattern->next ) {
+		for( pattern = patterns ; pattern != NULL ; pattern = pattern->next ) {
 			if( strcmp(name,pattern->name) == 0 )
 				break;
 		}
@@ -760,14 +780,14 @@ static retvalue gettargets(const char *listdir,struct update_origin *origins,str
 
 	updatetargets = NULL;
 
-	for( target = distribution->targets ; target ; target = target->next) {
+	for( target = distribution->targets ; target != NULL ; target = target->next) {
 		r = newupdatetarget(&updatetargets,target);
 		if( RET_WAS_ERROR(r) ) {
 			updates_freetargets(updatetargets);
 			return r;
 		}
 
-		for( origin = origins ; origin ; origin=origin->next ) {
+		for( origin = origins ; origin != NULL ; origin=origin->next ) {
 			if( origin->pattern == NULL )
 				r = adddeleteruletotarget(updatetargets);
 			else
@@ -798,7 +818,7 @@ static inline retvalue findmissingupdate(int count,const struct distribution *di
 			struct update_origin *u;
 
 			u = updates;
-			while( u && strcmp(u->pattern->name,update) != 0 ) 
+			while( u != NULL && strcmp(u->pattern->name,update) != 0 ) 
 				u = u->next;
 			if( u == NULL ) {
 				fprintf(stderr,"Update '%s' is listed in distribution '%s', but was not found!\n",update,distribution->codename);
@@ -823,7 +843,8 @@ retvalue updates_calcindices(const char *listdir,const struct update_pattern *pa
 	u_ds = NULL;
 	r = RET_OK;
 
-	for( distribution = distributions ; distribution ; distribution = distribution->next ) {
+	for( distribution = distributions ; distribution != NULL ; 
+			       distribution = distribution->next ) {
 		struct update_distribution *u_d;
 
 		u_d = calloc(1,sizeof(struct update_distribution));
@@ -846,7 +867,7 @@ retvalue updates_calcindices(const char *listdir,const struct update_pattern *pa
 			assert( u_d->origins != NULL );
 			last = u_d->origins;
 			count = 1;
-			while( last->next ) {
+			while( last->next != NULL ) {
 				last = last->next;
 				count++;
 			}
@@ -865,7 +886,7 @@ retvalue updates_calcindices(const char *listdir,const struct update_pattern *pa
 		*update_distributions = u_ds;
 	else
 		updates_freeupdatedistributions(u_ds);
-	return RET_OK;
+	return r;
 }
 
 static bool_t foundinorigins(struct update_origin *origins,size_t nameoffset,const char *name) {
@@ -905,24 +926,25 @@ static bool_t foundinindices(struct update_target *targets,size_t nameoffset,con
 static retvalue listclean_distribution(const char *listdir,DIR *dir, const char *pattern,
 		struct update_origin *origins,
 		struct update_target *targets) {
-	struct dirent entry, *r;
-	int e;
+	struct dirent *r;
 	size_t patternlen = strlen(pattern);
 	size_t nameoffset = strlen(listdir)+1;
 
-	while( 1 ) {
-		int namelen;
+	while( TRUE ) {
+		size_t namelen;
 		char *fullfilename;
+		int e;
 
-		e = readdir_r(dir,&entry,&r);
-		if( e != 0 ) {
+		r = readdir(dir);
+		if( r == NULL ) {
+			e = errno;
+			if( e == 0 )
+				return RET_OK;
 			/* this should not happen... */
 			e = errno;
 			fprintf(stderr,"Error reading dir '%s': %d=%m!\n",listdir,e);
 			return RET_ERRNO(e);
 		}
-		if( r == NULL || r != &entry )
-			return RET_OK;
 		namelen = _D_EXACT_NAMLEN(r);
 		if( namelen < patternlen || strncmp(pattern,r->d_name,patternlen) != 0)
 			continue;
@@ -990,6 +1012,7 @@ static inline retvalue startuporigin(struct aptmethodrun *run,struct update_orig
 
 	assert( origin != NULL && origin->pattern != NULL );
 	r = aptmethod_newmethod(run,origin->pattern->method,
+			origin->pattern->fallback,
 			origin->pattern->config,&method);
 	if( RET_WAS_ERROR(r) ) {
 		origin->download = NULL;
@@ -1007,12 +1030,17 @@ static retvalue updates_startup(struct aptmethodrun *run,struct update_distribut
 
 	result = RET_NOTHING;
 	for( d=distributions ; d != NULL ; d=d->next) {
-		if( d->distribution->deb_override || d->distribution->dsc_override 
-				|| d->distribution->udeb_override) {
+		if( d->distribution->deb_override != NULL || 
+		    d->distribution->dsc_override != NULL ||
+		    d->distribution->udeb_override != NULL ) {
 			if( verbose >= 0 )
 				fprintf(stderr,"Warning: Override-Files of '%s' ignored as not yet supported while updating!\n",d->distribution->codename);
 		}
-		for( origin=d->origins; origin; origin=origin->next ) {
+		if( d->distribution->tracking != dt_NONE ) {
+			fprintf(stderr,"Error(%s): Coping with trackingdata while updating not yet implemented!\n",d->distribution->codename);
+			return RET_ERROR;
+		}
+		for( origin=d->origins; origin != NULL ; origin=origin->next ) {
 			if( origin->pattern == NULL)
 				continue;
 			r = startuporigin(run,origin);
@@ -1082,6 +1110,7 @@ static inline retvalue readchecksums(struct update_origin *origin) {
 	return r;
 }
 
+/* returns RET_NOTHING when nothing new will be retrieved */
 static inline retvalue queueindex(struct update_index *index,int force) {
 	const struct update_origin *origin = index->origin;
 	int i;
@@ -1110,6 +1139,10 @@ static inline retvalue queueindex(struct update_index *index,int force) {
 				r = aptmethod_queuefile(origin->download,
 					index->upstream,index->filename,
 					md5sum,NULL,NULL);
+			} else if( RET_IS_OK(r) ) {
+				/* file is already there, nothing has
+				 * to be done */
+				r = RET_NOTHING;
 			}
 			return r;
 		}
@@ -1132,7 +1165,7 @@ static retvalue updates_queuemetalists(struct update_distribution *distributions
 
 	result = RET_NOTHING;
 	for( d=distributions ; d != NULL ; d=d->next) {
-		for( origin=d->origins;origin; origin=origin->next ) {
+		for( origin=d->origins; origin != NULL ; origin=origin->next ) {
 			if( origin->pattern == NULL)
 				continue;
 			if( origin->pattern->ignorerelease )
@@ -1148,7 +1181,7 @@ static retvalue updates_queuemetalists(struct update_distribution *distributions
 	return result;
 }
 
-static retvalue updates_queuelists(struct update_distribution *distributions,int force) {
+static retvalue updates_queuelists(struct update_distribution *distributions,bool_t skipold,bool_t *anythingtodo,int force) {
 	retvalue result,r;
 	struct update_origin *origin;
 	struct update_target *target;
@@ -1158,7 +1191,7 @@ static retvalue updates_queuelists(struct update_distribution *distributions,int
 	result = RET_NOTHING;
 	for( d=distributions ; d != NULL ; d=d->next) {
 
-		for( origin=d->origins;origin; origin=origin->next ) {
+		for( origin=d->origins; origin != NULL ; origin=origin->next ) {
 			if( origin->pattern == NULL)
 				continue;
 			r = readchecksums(origin);
@@ -1172,12 +1205,19 @@ static retvalue updates_queuelists(struct update_distribution *distributions,int
 			}
 		}
 		for( target=d->targets; target!=NULL ; target=target->next ) {
+			target->nothingnew = skipold;
 			for( index=target->indices ; index!=NULL ; index=index->next ) {
 				if( index->origin == NULL )
 					continue;
 				r = queueindex(index,force);
+				if( RET_IS_OK(r) ) {
+					target->nothingnew = FALSE;
+					*anythingtodo = TRUE;
+				}
+				if( RET_WAS_ERROR(r) )
+					index->failed = TRUE;
 				RET_UPDATE(result,r);
-				if( RET_WAS_ERROR(r) && ! force )
+				if( RET_WAS_ERROR(r) && force <= 0 )
 					return r;
 			}
 		}
@@ -1185,7 +1225,7 @@ static retvalue updates_queuelists(struct update_distribution *distributions,int
 	return result;
 }
 
-retvalue calllisthook(const char *listhook,struct update_index *index) {
+static retvalue calllisthook(const char *listhook,struct update_index *index) {
 	char *newfilename;
 	pid_t f,c;
 	int status; 
@@ -1250,7 +1290,10 @@ static retvalue updates_calllisthooks(struct update_distribution *distributions,
 	for( d=distributions ; d != NULL ; d=d->next) {
 
 		for( target=d->targets; target!=NULL ; target=target->next ) {
-			for( index=target->indices ; index; index=index->next ) {
+			if( target->nothingnew )
+				continue;
+			for( index=target->indices ; index != NULL ; 
+						     index=index->next ) {
 				if( index->origin == NULL )
 					continue;
 				if( index->origin->pattern->listhook == NULL )
@@ -1259,7 +1302,7 @@ static retvalue updates_calllisthooks(struct update_distribution *distributions,
 					continue;
 				}
 				r = calllisthook(index->origin->pattern->listhook,index);
-				if( RET_WAS_ERROR(r) && ! force ) {
+				if( RET_WAS_ERROR(r) && force <= 0 ) {
 					index->failed = TRUE;
 					return r;
 				}
@@ -1270,7 +1313,7 @@ static retvalue updates_calllisthooks(struct update_distribution *distributions,
 	return result;
 }
 
-upgrade_decision ud_decide_by_pattern(void *privdata, const char *package,UNUSED(const char *old_version),UNUSED(const char *new_version),const char *newcontrolchunk) {
+static upgrade_decision ud_decide_by_pattern(void *privdata, const char *package,UNUSED(const char *old_version),UNUSED(const char *new_version),const char *newcontrolchunk) {
 	struct update_pattern *pattern = privdata;
 	retvalue r;
 
@@ -1288,7 +1331,7 @@ upgrade_decision ud_decide_by_pattern(void *privdata, const char *package,UNUSED
 			break;
 	}
 
-	if( pattern->includecondition ) {
+	if( pattern->includecondition != NULL ) {
 		r = term_decidechunk(pattern->includecondition,newcontrolchunk);
 		if( RET_WAS_ERROR(r) )
 			return UD_ERROR;
@@ -1304,6 +1347,12 @@ static inline retvalue searchformissing(const char *dbdir,struct update_target *
 	struct update_index *index;
 	retvalue result,r;
 
+	if( u->nothingnew ) {
+		if( verbose >= 0 ) {
+		fprintf(stderr,"  nothing new for '%s' (use --noskipold to process anyway)\n",u->target->identifier);
+		}
+		return RET_NOTHING;
+	}
 	if( verbose > 2 )
 		fprintf(stderr,"  processing updates for '%s'\n",u->target->identifier);
 	r = upgradelist_initialize(&u->upgradelist,u->target,dbdir);
@@ -1312,14 +1361,14 @@ static inline retvalue searchformissing(const char *dbdir,struct update_target *
 
 	result = RET_NOTHING;
 
-	for( index=u->indices ; index ; index=index->next ) {
+	for( index=u->indices ; index != NULL ; index=index->next ) {
 
 		if( index->origin == NULL ) {
 			if( verbose > 4 )
 				fprintf(stderr,"  marking everything to be deleted\n");
 			r = upgradelist_deleteall(u->upgradelist);
 			RET_UPDATE(result,r);
-			if( RET_WAS_ERROR(r) && !force)
+			if( RET_WAS_ERROR(r) && force <= 0 )
 				return result;
 			u->ignoredelete = FALSE;
 			continue;
@@ -1334,7 +1383,7 @@ static inline retvalue searchformissing(const char *dbdir,struct update_target *
 
 		if( verbose > 4 )
 			fprintf(stderr,"  reading '%s'\n",index->filename);
-		assert(index->origin->download);
+		assert(index->origin->download!= NULL);
 		r = upgradelist_update(u->upgradelist,
 				index->origin->download,index->filename,
 				ud_decide_by_pattern,
@@ -1343,7 +1392,7 @@ static inline retvalue searchformissing(const char *dbdir,struct update_target *
 		if( RET_WAS_ERROR(r) )
 			u->ignoredelete = TRUE;
 		RET_UPDATE(result,r);
-		if( RET_WAS_ERROR(r) && !force)
+		if( RET_WAS_ERROR(r) && force <= 0 )
 			return result;
 	}
 	
@@ -1355,10 +1404,10 @@ static retvalue updates_readindices(const char *dbdir,struct update_distribution
 	struct update_target *u;
 
 	result = RET_NOTHING;
-	for( u=d->targets ; u ; u=u->next ) {
+	for( u=d->targets ; u != NULL ; u=u->next ) {
 		r = searchformissing(dbdir,u,force);
 		RET_UPDATE(result,r);
-		if( RET_WAS_ERROR(r) && !force )
+		if( RET_WAS_ERROR(r) && force <= 0 )
 			break;
 	}
 	return result;
@@ -1369,10 +1418,12 @@ static retvalue updates_enqueue(struct downloadcache *cache,filesdb filesdb,stru
 	struct update_target *u;
 
 	result = RET_NOTHING;
-	for( u=distribution->targets ; u ; u=u->next ) {
+	for( u=distribution->targets ; u != NULL ; u=u->next ) {
+		if( u->nothingnew )
+			continue;
 		r = upgradelist_enqueue(u->upgradelist,cache,filesdb,force);
 		RET_UPDATE(result,r);
-		if( RET_WAS_ERROR(r) && !force )
+		if( RET_WAS_ERROR(r) && force <= 0 )
 			break;
 	}
 	return result;
@@ -1384,12 +1435,14 @@ static retvalue updates_install(const char *dbdir,filesdb filesdb,references ref
 	struct update_target *u;
 
 	result = RET_NOTHING;
-	for( u=distribution->targets ; u ; u=u->next ) {
+	for( u=distribution->targets ; u != NULL ; u=u->next ) {
+		if( u->nothingnew )
+			continue;
 		r = upgradelist_install(u->upgradelist,dbdir,filesdb,refs,force,u->ignoredelete,dereferencedfilekeys);
 		RET_UPDATE(result,r);
 		upgradelist_free(u->upgradelist);
 		u->upgradelist = NULL;
-		if( RET_WAS_ERROR(r) && !force )
+		if( RET_WAS_ERROR(r) && force <= 0 )
 			break;
 	}
 	return result;
@@ -1398,13 +1451,13 @@ static retvalue updates_install(const char *dbdir,filesdb filesdb,references ref
 static void updates_dump(struct update_distribution *distribution) {
 	struct update_target *u;
 
-	for( u=distribution->targets ; u ; u=u->next ) {
+	for( u=distribution->targets ; u != NULL ; u=u->next ) {
 		printf("Updates needed for '%s':\n",u->target->identifier);
 		upgradelist_dump(u->upgradelist);
 	}
 }
 
-static retvalue updates_downloadlists(const char *methoddir,struct aptmethodrun *run,struct update_distribution *distributions, int force) {
+static retvalue updates_downloadlists(const char *methoddir,struct aptmethodrun *run,struct update_distribution *distributions, bool_t skipold, bool_t *anythingtodo, int force) {
 	retvalue r,result;
 
 	/* first get all "Release" and "Release.gpg" files */
@@ -1415,27 +1468,27 @@ static retvalue updates_downloadlists(const char *methoddir,struct aptmethodrun
 
 	r = aptmethod_download(run,methoddir,NULL);
 	RET_UPDATE(result,r);
-	if( RET_WAS_ERROR(r) && !force ) {
+	if( RET_WAS_ERROR(r) && force <= 0 ) {
 		return result;
 	}
 
 	/* Then get all index files (with perhaps md5sums from the above) */
-	r = updates_queuelists(distributions,force);
+	r = updates_queuelists(distributions,skipold,anythingtodo,force);
 	RET_UPDATE(result,r);
-	if( RET_WAS_ERROR(result) && !force ) {
+	if( RET_WAS_ERROR(result) && force <= 0 ) {
 		return result;
 	}
 
 	r = aptmethod_download(run,methoddir,NULL);
 	RET_UPDATE(result,r);
-	if( RET_WAS_ERROR(r) && !force ) {
+	if( RET_WAS_ERROR(r) && force <= 0 ) {
 		return result;
 	}
 
 	return result;
 }
 
-retvalue updates_update(const char *dbdir,const char *methoddir,filesdb filesdb,references refs,struct update_distribution *distributions,int force,bool_t nolistsdownload,struct strlist *dereferencedfilekeys) {
+retvalue updates_update(const char *dbdir,const char *methoddir,filesdb filesdb,references refs,struct update_distribution *distributions,int force,bool_t nolistsdownload,bool_t skipold,struct strlist *dereferencedfilekeys) {
 	retvalue result,r;
 	struct update_distribution *d;
 	struct aptmethodrun *run;
@@ -1445,6 +1498,13 @@ retvalue updates_update(const char *dbdir,const char *methoddir,filesdb filesdb,
 	if( RET_WAS_ERROR(r) )
 		return r;
 
+	if( nolistsdownload ) {
+		if( skipold && verbose >= 0 ) {
+			fprintf(stderr,"Ignoring --skipold because of --nolistsdownload\n");
+		}
+		skipold = FALSE;
+	}
+
 	/* preperations */
 	result = updates_startup(run,distributions,force);
 	if( RET_WAS_ERROR(result) && force <= 0 ) {
@@ -1455,19 +1515,25 @@ retvalue updates_update(const char *dbdir,const char *methoddir,filesdb filesdb,
 		if( verbose >= 0 )
 			fprintf(stderr,"Warning: As --nolistsdownload is given, index files are NOT checked.\n");
 	} else {
-		r = updates_downloadlists(methoddir,run,distributions,force);
+		bool_t anythingtodo = !skipold;
+
+		r = updates_downloadlists(methoddir,run,distributions,skipold,&anythingtodo,force);
 		RET_UPDATE(result,r);
 		if( RET_WAS_ERROR(result) && force <= 0 ) {
 			aptmethod_shutdown(run);
 			return result;
 		}
+		if( !anythingtodo ) {
+			fprintf(stderr,"Nothing to do found. (Use --noskipold to force processing)\n");
+			return result;
+		}
 	}
 	/* Call ListHooks (if given) on the downloaded index files.
 	 * (This is done even when nolistsdownload is given, as otherwise
 	 *  the filename to look in is not changed) */
 	r = updates_calllisthooks(distributions,force);
 	RET_UPDATE(result,r);
-	if( RET_WAS_ERROR(result) && !force ) {
+	if( RET_WAS_ERROR(result) && force <= 0 ) {
 		return result;
 	}
 
@@ -1481,17 +1547,17 @@ retvalue updates_update(const char *dbdir,const char *methoddir,filesdb filesdb,
 		return result;
 	}
 	
-	for( d=distributions ; d ; d=d->next) {
+	for( d=distributions ; d != NULL ; d=d->next) {
 		r = updates_readindices(dbdir,d,force);
 		RET_UPDATE(result,r);
-		if( RET_WAS_ERROR(r) && ! force )
+		if( RET_WAS_ERROR(r) && force <= 0 )
 			break;
 		r = updates_enqueue(cache,filesdb,d,force);
 		RET_UPDATE(result,r);
-		if( RET_WAS_ERROR(r) && ! force )
+		if( RET_WAS_ERROR(r) && force <= 0 )
 			break;
 	}
-	if( RET_WAS_ERROR(result) && ! force ) {
+	if( RET_WAS_ERROR(result) && force <= 0 ) {
 		aptmethod_shutdown(run);
 		return result;
 	}
@@ -1508,23 +1574,23 @@ retvalue updates_update(const char *dbdir,const char *methoddir,filesdb filesdb,
 	r = aptmethod_shutdown(run);
 	RET_UPDATE(result,r);
 
-	if( RET_WAS_ERROR(result) && !force ) {
+	if( RET_WAS_ERROR(result) && force <= 0 ) {
 		return result;
 	}
 	if( verbose >= 0 )
 		fprintf(stderr,"Installing (and possibly deleting) packages...\n");
 
-	for( d=distributions ; d ; d=d->next) {
+	for( d=distributions ; d != NULL ; d=d->next) {
 		r = updates_install(dbdir,filesdb,refs,d,force,dereferencedfilekeys);
 		RET_UPDATE(result,r);
-		if( RET_WAS_ERROR(r) && ! force )
+		if( RET_WAS_ERROR(r) && force <= 0 )
 			break;
 	}
 
 	return result;
 }
 
-retvalue updates_checkupdate(const char *dbdir,const char *methoddir,struct update_distribution *distributions,int force,bool_t nolistsdownload) {
+retvalue updates_checkupdate(const char *dbdir,const char *methoddir,struct update_distribution *distributions,int force,bool_t nolistsdownload,bool_t skipold) {
 	struct update_distribution *d;
 	retvalue result,r;
 	struct aptmethodrun *run;
@@ -1533,6 +1599,13 @@ retvalue updates_checkupdate(const char *dbdir,const char *methoddir,struct upda
 	if( RET_WAS_ERROR(r) )
 		return r;
 
+	if( nolistsdownload ) {
+		if( skipold && verbose >= 0 ) {
+			fprintf(stderr,"Ignoring --skipold because of --nolistsdownload\n");
+		}
+		skipold = FALSE;
+	}
+
 	result = updates_startup(run,distributions,force);
 	if( RET_WAS_ERROR(result) && force <= 0 ) {
 		aptmethod_shutdown(run);
@@ -1542,26 +1615,31 @@ retvalue updates_checkupdate(const char *dbdir,const char *methoddir,struct upda
 		if( verbose >= 0 )
 			fprintf(stderr,"Warning: As --nolistsdownload is given, index files are NOT checked.\n");
 	} else {
-		r = updates_downloadlists(methoddir,run,distributions,force);
+		bool_t anythingtodo = !skipold;
+		r = updates_downloadlists(methoddir,run,distributions,skipold,&anythingtodo,force);
 		RET_UPDATE(result,r);
 		if( RET_WAS_ERROR(result) && force <= 0 ) {
 			aptmethod_shutdown(run);
 			return result;
 		}
+		if( !anythingtodo ) {
+			fprintf(stderr,"Nothing to do found. (Use --noskipold to force processing)\n");
+			return result;
+		}
 	}
 	/* Call ListHooks (if given) on the downloaded index files.
 	 * (This is done even when nolistsdownload is given, as otherwise
 	 *  the filename to look in is not changed) */
 	r = updates_calllisthooks(distributions,force);
 	RET_UPDATE(result,r);
-	if( RET_WAS_ERROR(result) && !force ) {
+	if( RET_WAS_ERROR(result) && force <= 0 ) {
 		return result;
 	}
 	if( verbose > 0 )
 		fprintf(stderr,"Shutting down aptmethods...\n");
 	r = aptmethod_shutdown(run);
 	RET_UPDATE(result,r);
-	if( RET_WAS_ERROR(result) && !force ) {
+	if( RET_WAS_ERROR(result) && force <= 0 ) {
 		return result;
 	}
 
@@ -1569,10 +1647,10 @@ retvalue updates_checkupdate(const char *dbdir,const char *methoddir,struct upda
 	if( verbose >= 0 )
 		fprintf(stderr,"Calculating packages to get...\n");
 	
-	for( d=distributions ; d ; d=d->next) {
+	for( d=distributions ; d != NULL ; d=d->next) {
 		r = updates_readindices(dbdir,d,force);
 		RET_UPDATE(result,r);
-		if( RET_WAS_ERROR(r) && ! force )
+		if( RET_WAS_ERROR(r) && force <= 0 )
 			break;
 		updates_dump(d);
 	}
@@ -1580,7 +1658,7 @@ retvalue updates_checkupdate(const char *dbdir,const char *methoddir,struct upda
 	return result;
 }
 
-static retvalue singledistributionupdate(const char *dbdir,const char *methoddir,filesdb filesdb,references refs,struct update_distribution *d,int force,bool_t nolistsdownload,struct strlist *dereferencedfilekeys) {
+static retvalue singledistributionupdate(const char *dbdir,const char *methoddir,filesdb filesdb,references refs,struct update_distribution *d,int force,bool_t nolistsdownload,bool_t skipold, struct strlist *dereferencedfilekeys) {
 	struct aptmethodrun *run;
 	struct downloadcache *cache;
 	struct update_origin *origin;
@@ -1594,7 +1672,7 @@ static retvalue singledistributionupdate(const char *dbdir,const char *methoddir
 		return r;
 
 	/* preperations */
-	for( origin=d->origins; origin; origin=origin->next ) {
+	for( origin=d->origins; origin != NULL ; origin=origin->next ) {
 		if( origin->pattern == NULL)
 			continue;
 		r = startuporigin(run,origin);
@@ -1605,7 +1683,7 @@ static retvalue singledistributionupdate(const char *dbdir,const char *methoddir
 		}
 	}
 	if( !nolistsdownload ) {
-		for( origin=d->origins;origin; origin=origin->next ) {
+		for( origin=d->origins; origin != NULL ; origin=origin->next ) {
 			if( origin->pattern == NULL)
 				continue;
 			if( origin->pattern->ignorerelease )
@@ -1622,11 +1700,11 @@ static retvalue singledistributionupdate(const char *dbdir,const char *methoddir
 		}
 		r = aptmethod_download(run,methoddir,NULL);
 		RET_UPDATE(result,r);
-		if( RET_WAS_ERROR(r) && !force ) {
+		if( RET_WAS_ERROR(r) && force <= 0 ) {
 			aptmethod_shutdown(run);
 			return r;
 		}
-		for( origin=d->origins;origin; origin=origin->next ) {
+		for( origin=d->origins; origin != NULL ; origin=origin->next ) {
 			if( origin->pattern == NULL)
 				continue;
 			r = readchecksums(origin);
@@ -1643,6 +1721,7 @@ static retvalue singledistributionupdate(const char *dbdir,const char *methoddir
 
 	for( target=d->targets; target!=NULL ; target=target->next ) {
 		struct update_index *index;
+		target->nothingnew = skipold;
 		if( !nolistsdownload ) {
 			for( index=target->indices ; index!=NULL ; index=index->next ) {
 				if( index->origin == NULL || index->origin->failed )
@@ -1655,8 +1734,12 @@ static retvalue singledistributionupdate(const char *dbdir,const char *methoddir
 						return r;
 					}
 					index->failed = TRUE;
+				} else if( RET_IS_OK(r) ) {
+					target->nothingnew = FALSE;
 				}
 			}
+			if( target->nothingnew )
+				continue;
 			r = aptmethod_download(run,methoddir,NULL);
 			RET_UPDATE(result,r);
 			if( RET_WAS_ERROR(r) && force <= 0 ) {
@@ -1665,7 +1748,7 @@ static retvalue singledistributionupdate(const char *dbdir,const char *methoddir
 			}
 		}
 
-		for( index=target->indices ; index; index=index->next ) {
+		for( index=target->indices ; index != NULL ; index=index->next ) {
 			if( index->origin == NULL )
 				continue;
 			if( index->origin->pattern->listhook == NULL )
@@ -1699,6 +1782,10 @@ static retvalue singledistributionupdate(const char *dbdir,const char *methoddir
 			continue;
 		}
 		r = searchformissing(dbdir,target,force);
+		if( r == RET_NOTHING ) {
+			(void)downloadcache_free(cache);
+			continue;
+		}
 		RET_UPDATE(result,r);
 		if( RET_WAS_ERROR(r) && force <= 0 ) {
 			(void)downloadcache_free(cache);
@@ -1743,7 +1830,7 @@ static retvalue singledistributionupdate(const char *dbdir,const char *methoddir
 	return result;
 }
 
-retvalue updates_iteratedupdate(const char *confdir,const char *dbdir,const char *distdir,const char *methoddir,filesdb filesdb,references refs,struct update_distribution *distributions,int force,bool_t nolistsdownload,struct strlist *dereferencedfilekeys) {
+retvalue updates_iteratedupdate(const char *confdir,const char *dbdir,const char *distdir,const char *methoddir,filesdb filesdb,references refs,struct update_distribution *distributions,int force,bool_t nolistsdownload,bool_t skipold,struct strlist *dereferencedfilekeys) {
 	retvalue result,r;
 	struct update_distribution *d;
 
@@ -1754,15 +1841,16 @@ retvalue updates_iteratedupdate(const char *confdir,const char *dbdir,const char
 
 	result = RET_NOTHING;
 	for( d=distributions ; d != NULL ; d=d->next) {
-		if( d->distribution->deb_override || d->distribution->dsc_override 
-				|| d->distribution->udeb_override) {
+		if( d->distribution->deb_override != NULL || 
+		    d->distribution->dsc_override != NULL ||
+		    d->distribution->udeb_override!= NULL ) {
 			if( verbose >= 0 )
 				fprintf(stderr,"Warning: Override-Files of '%s' ignored as not yet supported while updating!\n",d->distribution->codename);
 		}
-		r = singledistributionupdate(dbdir,methoddir,filesdb,refs,d,force,nolistsdownload,dereferencedfilekeys);
+		r = singledistributionupdate(dbdir,methoddir,filesdb,refs,d,force,nolistsdownload,skipold,dereferencedfilekeys);
 		RET_UPDATE(result,r);
 		if( RET_IS_OK(r) ) {
-			r = distribution_export(d->distribution,confdir,dbdir,distdir,force,TRUE);
+			r = distribution_export(d->distribution,confdir,dbdir,distdir,TRUE);
 			RET_UPDATE(result,r);
 		}
 	}
